@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2010 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -71,7 +71,7 @@
 #include "light.h"
 #include "global.h"
 #include "ui.h"
-#include "video.h"
+#include "tachyon_video.h"
 #include "objbound.h"
 #include "grid.h"
 
@@ -96,8 +96,6 @@ void renderscene(scenedef scene) {
   outfile = opentgafile(scene.outfilename);
   */
 
-  timerstart();
-
   trace_region (scene, 0/*outfile*/, 0, 0, scene.hres, scene.vres);
   //fclose((FILE *)outfile);
 } /* end of renderscene() */
