@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2010 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -68,7 +68,7 @@
 #include "intersect.h"
 #include "global.h"
 #include "ui.h"
-#include "video.h"
+#include "tachyon_video.h"
 
 // shared but read-only so could be private too
 static thr_parms *all_parms;
