@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2010 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -30,38 +30,26 @@
     This file contains a few implementations, so it may look overly complicated.
     The most efficient implementation is also separated into convex_hull_sample.cpp
 */
+#include <cassert>
 #include "convex_hull.h"
 
 typedef util::point<double> point_t;
 
-#define USETBB      1
-#define USECONCVEC  1
-#define INIT_ONCE   1
+#ifndef USETBB
+    #define USETBB      1
+#endif
+#ifndef USECONCVEC
+    #define USECONCVEC   1
+#endif
 
 #if !USETBB // Serial implementation of Quick Hull algorithm
 
 typedef std::vector< point_t > pointVec_t;
 
+void serial_initialize(pointVec_t &points);
+
 // C++ style serial code
 
-class FillRNDPointsVector : public std::unary_function<point_t&, void> {
-    unsigned int rseed;
-    size_t       count;
-public:
-    FillRNDPointsVector() : rseed(1), count(0) {}
-
-    void operator()(point_t& p) {
-        p = util::GenerateRNDPoint<double>(count, rseed);
-    }
-};
-
-void initialize(pointVec_t &points) {
-    points.clear();
-    points.resize(cfg::MAXPOINTS);
-
-    std::for_each(points.begin(), points.end(), FillRNDPointsVector());
-}
-
 class FindXExtremum : public std::unary_function<const point_t&, void> {
 public:
     typedef enum {
@@ -91,6 +79,7 @@ private:
         case maxX:
             return p.x>extrXPoint.x; break;
         }
+        return false; // avoid warning
     }
 };
 
@@ -133,7 +122,7 @@ point_t divide(const pointVec_t &P, pointVec_t &P_reduced, const point_t &p1, co
     SplitByCP splitByCP(p1, p2, P_reduced);
     point_t farPoint = std::for_each(P.begin(), P.end(), splitByCP);
 
-    if(util::VERBOSE) {
+    if(util::verbose) {
         std::stringstream ss;
         ss << P.size() << " nodes in bucket"<< ", "
             << "dividing by: [ " << p1 << ", " << p2 << " ], "
@@ -145,17 +134,15 @@ point_t divide(const pointVec_t &P, pointVec_t &P_reduced, const point_t &p1, co
 }
 
 void divide_and_conquer(const pointVec_t &P, pointVec_t &H, point_t p1, point_t p2) {
-    if (P.size()<2) {
+    assert(P.size() >= 2);
+    pointVec_t P_reduced;
+    pointVec_t H1, H2;
+    point_t p_far = divide(P, P_reduced, p1, p2);
+    if (P_reduced.size()<2) {
         H.push_back(p1);
-        H.insert(H.end(), P.begin(), P.end());
+        H.insert(H.end(), P_reduced.begin(), P_reduced.end());
     }
     else {
-        pointVec_t P_reduced;
-        pointVec_t H1, H2;
-        point_t p_far;
-
-        p_far = divide(P, P_reduced, p1, p2);
-
         divide_and_conquer(P_reduced, H1, p1, p_far);
         divide_and_conquer(P_reduced, H2, p_far, p2);
 
@@ -165,8 +152,10 @@ void divide_and_conquer(const pointVec_t &P, pointVec_t &H, point_t p1, point_t
 }
 
 void quickhull(const pointVec_t &points, pointVec_t &hull) {
-    hull.clear();
-
+    if (points.size() < 2) {
+        hull.insert(hull.end(), points.begin(), points.end());
+        return;
+    }
     point_t p_maxx = extremum<FindXExtremum::maxX>(points);
     point_t p_minx = extremum<FindXExtremum::minX>(points);
 
@@ -188,13 +177,13 @@ int main(int argc, char* argv[]) {
     std::cout << "Starting serial version of QUICK HULL algorithm" << std::endl;
 
     tm_init = util::gettime();
-    initialize(points);
+    serial_initialize(points);
+    tm_start = util::gettime();
+    std::cout << "Init time: " << util::time_diff(tm_init, tm_start) << "  Points in input: " << points.size() << "\n";
     tm_start = util::gettime();
     quickhull(points, hull);
     tm_end = util::gettime();
-
-    util::WriteResults(1, util::time_diff(tm_init, tm_start),
-        util::time_diff(tm_start, tm_end));
+    std::cout << "Serial time: " << util::time_diff(tm_start, tm_end) << "  Points in hull: " << hull.size() << "\n";
 }
 
 #else // USETBB - parallel version of Quick Hull algorithm
@@ -219,6 +208,9 @@ void appendVector(const pointVec_t& src, pointVec_t& dest) {
     std::copy(src.begin(), src.end(), dest.grow_by(src.size()));
 }
 
+void grow_vector_to_at_least(pointVec_t& vect, size_t size) {
+    vect.grow_to_at_least(size);
+}
 #else // USE STD::VECTOR - include spin_mutex.h and lock vector operations
 #include "tbb/spin_mutex.h"
 
@@ -236,65 +228,67 @@ void appendVector(mutex_t& insertMutex, const point_t* src, size_t srcSize,
     dest.insert(dest.end(), src, src + srcSize);
 }
 
+void grow_vector_to_at_least(mutex_t& mutex, pointVec_t& vect, size_t size) {
+    mutex_t::scoped_lock lock(mutex);
+    if (vect.size()< size){
+        vect.resize(size);
+    }
+}
 #endif // USECONCVEC
 
 class FillRNDPointsVector {
     pointVec_t          &points;
-    mutable unsigned int rseed;
 public:
-    static const size_t  grainSize = cfg::GENERATE_GS;
+    static const size_t  grainSize = cfg::generateGrainSize;
 #if !USECONCVEC
     static mutex_t       pushBackMutex;
 #endif // USECONCVEC
-    FillRNDPointsVector(pointVec_t& _points)
-        : points(_points), rseed(1) {}
 
-    FillRNDPointsVector(const FillRNDPointsVector& other)
-        : points(other.points), rseed(other.rseed+1) {}
+    explicit FillRNDPointsVector(pointVec_t& _points)
+        : points(_points){}
 
     void operator()(const range_t& range) const {
+        util::rng the_rng(range.begin());
         const size_t i_end = range.end();
         size_t count = 0;
-        for(size_t i = range.begin(); i != i_end; ++i) {
 #if USECONCVEC
-            points.push_back(util::GenerateRNDPoint<double>(count, rseed));
-#else // Locked push_back to a not thread-safe STD::VECTOR
-            {
-                mutex_t::scoped_lock lock(pushBackMutex);
-                points.push_back(util::GenerateRNDPoint<double>(count, rseed));
-            }
+            points.grow_to_at_least(i_end);
+#else // Locked enlarge to a not thread-safe STD::VECTOR
+            grow_vector_to_at_least(pushBackMutex,points,i_end);
 #endif // USECONCVEC
+
+        for(size_t i = range.begin(); i != i_end; ++i) {
+            points[i]=util::GenerateRNDPoint<double>(count,the_rng,util::rng::max_rand);
         }
     }
 };
 
 class FillRNDPointsVector_buf {
     pointVec_t          &points;
-    mutable unsigned int rseed;
 public:
-    static const size_t  grainSize = cfg::GENERATE_GS;
+    static const size_t  grainSize = cfg::generateGrainSize;
 #if !USECONCVEC
     static mutex_t       insertMutex;
 #endif // USECONCVEC
 
-    FillRNDPointsVector_buf(pointVec_t& _points)
-        : points(_points), rseed(1) {}
-
-    FillRNDPointsVector_buf(const FillRNDPointsVector_buf& other)
-        : points(other.points), rseed(other.rseed+1) {}
+    explicit FillRNDPointsVector_buf(pointVec_t& _points)
+        : points(_points){}
 
     void operator()(const range_t& range) const {
+        util::rng the_rng(range.begin());
         const size_t i_end = range.end();
         size_t count = 0, j = 0;
         point_t tmp_vec[grainSize];
+
         for(size_t i=range.begin(); i!=i_end; ++i) {
-            tmp_vec[j++] = util::GenerateRNDPoint<double>(count, rseed);
+            tmp_vec[j++] = util::GenerateRNDPoint<double>(count,the_rng,util::rng::max_rand);
         }
 #if USECONCVEC
-        appendVector(tmp_vec, j, points);
+        grow_vector_to_at_least(points,range.end());
 #else // USE STD::VECTOR
-        appendVector(insertMutex, tmp_vec, j, points);
+        grow_vector_to_at_least(insertMutex,points,range.end());
 #endif // USECONCVEC
+        std::copy(tmp_vec, tmp_vec+j,points.begin()+range.begin());
     }   
 };
 
@@ -305,13 +299,23 @@ mutex_t FillRNDPointsVector_buf::insertMutex = mutex_t();
 
 template<typename BodyType>
 void initialize(pointVec_t &points) {
-    points.clear();
+    //This function generate the same series of point on every call.
+    //Reproducibility is needed for benchmarking to produce reliable results.
+    //It is achieved through the following points:
+    //      - FillRNDPointsVector_buf instance has its own local instance
+    //        of random number generator, which in turn does not use any global data
+    //      - tbb::simple_partitioner produce the same set of ranges on every call to
+    //        tbb::parallel_for
+    //      - local RNG instances are seeded by the starting indexes of corresponding ranges
+    //      - grow_to_at_least() enables putting points into the resulting vector in deterministic order
+    //        (unlike concurrent push_back or grow_by).
 
     // In the buffered version, a temporary storage for as much as grainSize elements 
     // is allocated inside the body. Since auto_partitioner may increase effective
     // range size which would cause a crash, simple partitioner has to be used.
-    tbb::parallel_for(range_t(0, cfg::MAXPOINTS, BodyType::grainSize),
-                      BodyType(points), tbb::simple_partitioner());
+
+    tbb::parallel_for(range_t(0, cfg::numberOfPoints, BodyType::grainSize),
+    BodyType(points), tbb::simple_partitioner());
 }
 
 class FindXExtremum {
@@ -320,7 +324,7 @@ public:
         minX, maxX
     } extremumType;
 
-    static const size_t  grainSize = cfg::FINDEXT_GS;
+    static const size_t  grainSize = cfg::findExtremumGrainSize;
 
     FindXExtremum(const pointVec_t& points_, extremumType exType_)
         : points(points_), exType(exType_), extrXPoint(points[0]) {}
@@ -378,7 +382,7 @@ class SplitByCP {
     point_t              farPoint;
     double               howFar;
 public:
-    static const size_t grainSize = cfg::DIVIDE_GS;
+    static const size_t grainSize = cfg::divideGrainSize;
 #if !USECONCVEC
     static mutex_t      pushBackMutex;
 #endif // USECONCVEC
@@ -438,7 +442,7 @@ class SplitByCP_buf {
     point_t              farPoint;
     double               howFar;
 public:
-    static const size_t  grainSize = cfg::DIVIDE_GS;
+    static const size_t  grainSize = cfg::divideGrainSize;
 #if !USECONCVEC
     static mutex_t       insertMutex;
 #endif // USECONCVEC
@@ -504,7 +508,7 @@ point_t divide(const pointVec_t &P, pointVec_t &P_reduced,
     tbb::parallel_reduce(range_t(0, P.size(), BodyType::grainSize),
                          body, tbb::simple_partitioner() );
 
-    if(util::VERBOSE) {
+    if(util::verbose) {
         std::stringstream ss;
         ss << P.size() << " nodes in bucket"<< ", "
             << "dividing by: [ " << p1 << ", " << p2 << " ], "
@@ -517,25 +521,26 @@ point_t divide(const pointVec_t &P, pointVec_t &P_reduced,
 
 void divide_and_conquer(const pointVec_t &P, pointVec_t &H,
                         point_t p1, point_t p2, bool buffered) {
-    if (P.size()<2) {
+    assert(P.size() >= 2);
+    pointVec_t P_reduced;
+    pointVec_t H1, H2;
+    point_t p_far;
+    
+    if(buffered) {
+        p_far = divide<SplitByCP_buf>(P, P_reduced, p1, p2);
+    } else {
+        p_far = divide<SplitByCP>(P, P_reduced, p1, p2);
+    }
+
+    if (P_reduced.size()<2) {
         H.push_back(p1);
 #if USECONCVEC
-        appendVector(P, H);
+        appendVector(P_reduced, H);
 #else // insert into STD::VECTOR
-        H.insert(H.end(), P.begin(), P.end());
+        H.insert(H.end(), P_reduced.begin(), P_reduced.end());
 #endif
     }
     else {
-        pointVec_t P_reduced;
-        pointVec_t H1, H2;
-        point_t p_far;
-
-        if(buffered) {
-            p_far = divide<SplitByCP_buf>(P, P_reduced, p1, p2);
-        } else {
-            p_far = divide<SplitByCP>(P, P_reduced, p1, p2);
-        }
-
         divide_and_conquer(P_reduced, H1, p1, p_far, buffered);
         divide_and_conquer(P_reduced, H2, p_far, p2, buffered);
 
@@ -550,7 +555,14 @@ void divide_and_conquer(const pointVec_t &P, pointVec_t &H,
 }
 
 void quickhull(const pointVec_t &points, pointVec_t &hull, bool buffered) {
-    hull.clear();
+    if (points.size() < 2) {
+#if USECONCVEC
+        appendVector(points, hull);
+#else // STD::VECTOR
+        hull.insert(hull.end(), points.begin(), points.end());
+#endif // USECONCVEC
+        return;
+    }
 
     point_t p_maxx = extremum<FindXExtremum::maxX>(points);
     point_t p_minx = extremum<FindXExtremum::minX>(points);
@@ -569,73 +581,66 @@ void quickhull(const pointVec_t &points, pointVec_t &hull, bool buffered) {
 int main(int argc, char* argv[]) {
     util::ParseInputArgs(argc, argv);
 
-    pointVec_t      points;
-    pointVec_t      hull;
     int             nthreads;
     util::my_time_t tm_init, tm_start, tm_end;
-    pointVec_t      tmp_points;
 
 #if USECONCVEC
-    std::cout << "Starting TBB unbufferred push_back version of QUICK HULL algorithm" << std::endl;
+    std::cout << "Starting TBB unbuffered push_back version of QUICK HULL algorithm" << std::endl;
 #else
-    std::cout << "Starting STL locked unbufferred push_back version of QUICK HULL algorithm" << std::endl;
+    std::cout << "Starting STL locked unbuffered push_back version of QUICK HULL algorithm" << std::endl;
 #endif // USECONCVEC
 
-    for(nthreads=cfg::NUM_THREADS_START; nthreads<=cfg::NUM_THREADS_END;
+    for(nthreads=cfg::threads.first; nthreads<=cfg::threads.last;
         ++nthreads) {
+        pointVec_t      points;
+        pointVec_t      hull;
+
         tbb::task_scheduler_init init(nthreads);
-#if INIT_ONCE
-        if(nthreads==cfg::NUM_THREADS_START) {
-            tm_init = util::gettime();
-            initialize<FillRNDPointsVector>(points);
-        }
-        else /* timing generation for stats, but use original data set */ {
-            tm_init = util::gettime();
-            initialize<FillRNDPointsVector>(tmp_points);
-        }
-#else
         tm_init = util::gettime();
         initialize<FillRNDPointsVector>(points);
-#endif // INIT_ONCE
+        tm_start = util::gettime();
+        std::cout << "Parallel init time on " << nthreads << " threads: " << util::time_diff(tm_init, tm_start) << "  Points in input: " << points.size() << "\n";
+
         tm_start = util::gettime();
         quickhull(points, hull, false);
         tm_end = util::gettime();
-
-        util::WriteResults(nthreads, util::time_diff(tm_init, tm_start),
-            util::time_diff(tm_start, tm_end));
+        std::cout << "Time on " << nthreads << " threads: " << util::time_diff(tm_start, tm_end) << "  Points in hull: " << hull.size() << "\n";
     }
 
 #if USECONCVEC 
-    std::cout << "Starting TBB bufferred version of QUICK HULL algorithm" << std::endl;
+    std::cout << "Starting TBB buffered version of QUICK HULL algorithm" << std::endl;
 #else
-    std::cout << "Starting STL locked bufferred version of QUICK HULL algorithm" << std::endl;
+    std::cout << "Starting STL locked buffered version of QUICK HULL algorithm" << std::endl;
 #endif
 
-    for(nthreads=cfg::NUM_THREADS_START; nthreads<=cfg::NUM_THREADS_END;
+    for(nthreads=cfg::threads.first; nthreads<=cfg::threads.last;
         ++nthreads) {
+        pointVec_t      points;
+        pointVec_t      hull;
+
         tbb::task_scheduler_init init(nthreads);
-#if INIT_ONCE
-        if(nthreads==cfg::NUM_THREADS_START) {
-            tm_init = util::gettime();
-            initialize<FillRNDPointsVector_buf>(points);
-        }
-        else /* timing generation for stats, but use original data set */ {
-            tm_init = util::gettime();
-            initialize<FillRNDPointsVector_buf>(tmp_points);
-        }
-#else
+
         tm_init = util::gettime();
         initialize<FillRNDPointsVector_buf>(points);
-#endif // INIT_ONCE
+        tm_start = util::gettime();
+        std::cout << "Init time on " << nthreads << " threads: " << util::time_diff(tm_init, tm_start) << "  Points in input: " << points.size() << "\n";
+
         tm_start = util::gettime();
         quickhull(points, hull, true);
         tm_end = util::gettime();
-
-        util::WriteResults(nthreads, util::time_diff(tm_init, tm_start),
-            util::time_diff(tm_start, tm_end));
+        std::cout << "Time on " << nthreads << " threads: " << util::time_diff(tm_start, tm_end) << "  Points in hull: " << hull.size() << "\n";
     }    
 
     return 0;
 }
 
 #endif // USETBB
+
+void serial_initialize(pointVec_t &points) {
+    points.reserve(cfg::numberOfPoints);
+
+    unsigned int rseed=1;
+    for(size_t i=0, count=0; long(i)<cfg::numberOfPoints; ++i) {
+        points.push_back(util::GenerateRNDPoint<double>(count,&std::rand,RAND_MAX ));
+    }
+}
