@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2010 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -31,14 +31,13 @@
 
 #include "concurrent_vector.h"
 #include "tbb_thread.h"
+#include "tbb_allocator.h"
 #include "cache_aligned_allocator.h"
 #include "aligned_space.h"
-#if __SUNPRO_CC
 #include <string.h>  // for memcpy
-#endif
 
 #if _WIN32||_WIN64
-#include <windows.h>
+#include "machine/windows_api.h"
 #else
 #include <pthread.h>
 #endif
@@ -791,7 +790,6 @@ namespace interface6 {
 
         //! Returns reference to calling thread's local copy, creating one if necessary
         reference local(bool& exists)  {
-            __TBB_ASSERT(ETS_key_type==ets_no_key,"ets_key_per_instance not yet implemented"); 
             void* ptr = this->table_lookup(exists);
             return *(T*)ptr;
         }
