@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2010 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -27,17 +27,11 @@
 */
 
 #ifndef __TBB_machine_H
-#error Do not include this file directly; include tbb_machine.h instead
+#error Do not #include this internal file directly; use public TBB headers instead.
 #endif
 
-#include <stdint.h>
-#include <unistd.h>
 #include <sched.h>
-
-// Definition of __TBB_Yield()
-#ifndef __TBB_Yield
 #define __TBB_Yield()  sched_yield()
-#endif
 
 /* Futex definitions */
 #include <sys/syscall.h>
