@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2010 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -29,37 +29,19 @@
 #ifndef __TBB_parallel_reduce_H
 #define __TBB_parallel_reduce_H
 
+#include <new>
 #include "task.h"
 #include "aligned_space.h"
 #include "partitioner.h"
-#include <new>
+#include "tbb_profiling.h"
 
 namespace tbb {
 
+namespace interface6 {
 //! @cond INTERNAL
 namespace internal {
 
-    //! ITT instrumented routine that stores src into location pointed to by dst.
-    void __TBB_EXPORTED_FUNC itt_store_pointer_with_release_v3( void* dst, void* src );
-
-    //! ITT instrumented routine that loads pointer from location pointed to by src.
-    void* __TBB_EXPORTED_FUNC itt_load_pointer_with_acquire_v3( const void* src );
-
-    template<typename T> inline void parallel_reduce_store_body( T*& dst, T* src ) {
-#if TBB_USE_THREADING_TOOLS
-        itt_store_pointer_with_release_v3(&dst,src);
-#else
-        __TBB_store_with_release(dst,src);
-#endif /* TBB_USE_THREADING_TOOLS */
-    }
-
-    template<typename T> inline T* parallel_reduce_load_body( T*& src ) {
-#if TBB_USE_THREADING_TOOLS
-        return static_cast<T*>(itt_load_pointer_with_acquire_v3(&src));
-#else
-        return __TBB_load_with_acquire(src);
-#endif /* TBB_USE_THREADING_TOOLS */
-    }
+    using namespace tbb::internal;
 
     //! 0 if root, 1 if a left child, 2 if a right child.
     /** Represented as a char, not enum, for compactness. */
@@ -68,16 +50,16 @@ namespace internal {
     //! Task type use to combine the partial results of parallel_reduce.
     /** @ingroup algorithms */
     template<typename Body>
-    class finish_reduce: public task {
+    class finish_reduce: public flag_task {
         //! Pointer to body, or NULL if the left child has not yet finished. 
-        Body* my_body;
         bool has_right_zombie;
         const reduction_context my_context;
+        Body* my_body;
         aligned_space<Body,1> zombie_space;
-        finish_reduce( char context_ ) : 
-            my_body(NULL),
-            has_right_zombie(false),
-            my_context(context_)
+        finish_reduce( reduction_context context_ ) : 
+            has_right_zombie(false), // TODO: substitute by flag_task::child_stolen?
+            my_context(context_),
+            my_body(NULL)
         {
         }
         task* execute() {
@@ -87,10 +69,10 @@ namespace internal {
                 my_body->join( *s );
                 s->~Body();
             }
-            if( my_context==1 ) 
-                parallel_reduce_store_body( static_cast<finish_reduce*>(parent())->my_body, my_body );
+            if( my_context==1 )  // left child
+                itt_store_word_with_release( static_cast<finish_reduce*>(parent())->my_body, my_body );
             return NULL;
-        }       
+        }
         template<typename Range,typename Body_, typename Partitioner>
         friend class start_reduce;
     };
@@ -102,12 +84,13 @@ namespace internal {
         typedef finish_reduce<Body> finish_type;
         Body* my_body;
         Range my_range;
-        typename Partitioner::partition_type my_partition;
-        reduction_context my_context;
+        typename Partitioner::task_partition_type my_partition;
+        reduction_context my_context; // TODO: factor out into start_reduce_base
         /*override*/ task* execute();
         template<typename Body_>
         friend class finish_reduce;
     
+public:
         //! Constructor used for root task
         start_reduce( const Range& range, Body* body, Partitioner& partitioner ) :
             my_body(body),
@@ -117,7 +100,7 @@ namespace internal {
         {
         }
         //! Splitting constructor used to generate children.
-        /** this becomes left child.  Newly constructed object is right child. */
+        /** parent_ becomes left child.  Newly constructed object is right child. */
         start_reduce( start_reduce& parent_, split ) :
             my_body(parent_.my_body),
             my_range(parent_.my_range,split()),
@@ -127,12 +110,22 @@ namespace internal {
             my_partition.set_affinity(*this);
             parent_.my_context = 1;
         }
+        //! Construct right child from the given range as response to the demand.
+        /** parent_ remains left child.  Newly constructed object is right child. */
+        start_reduce( start_reduce& parent_, const Range& r, depth_t d ) :
+            my_body(parent_.my_body),
+            my_range(r),
+            my_partition(parent_.my_partition,split()),
+            my_context(2) // right leaf mark
+        {
+            my_partition.set_affinity(*this);
+            my_partition.align_depth( d );
+            parent_.my_context = 1; // left leaf mark
+        }
         //! Update affinity info, if any
         /*override*/ void note_affinity( affinity_id id ) {
             my_partition.note_affinity( id );
         }
-
-public:
         static void run( const Range& range, Body& body, Partitioner& partitioner ) {
             if( !range.empty() ) {
 #if !__TBB_TASK_GROUP_CONTEXT || TBB_JOIN_OUTER_TASK_GROUP
@@ -151,33 +144,122 @@ public:
                 task::spawn_root_and_wait( *new(task::allocate_root(context)) start_reduce(range,&body,partitioner) );
         }
 #endif /* __TBB_TASK_GROUP_CONTEXT */
+        //! create a continuation task, serve as callback for partitioner
+        finish_type *create_continuation() {
+            return new( allocate_continuation() ) finish_type(my_context);
+        }
+        //! Run body for range
+        void run_body( Range &r ) { (*my_body)( r ); }
     };
-
     template<typename Range, typename Body, typename Partitioner>
     task* start_reduce<Range,Body,Partitioner>::execute() {
-        if( my_context==2 ) {
-            finish_type* p = static_cast<finish_type*>(parent() );
-            if( !parallel_reduce_load_body(p->my_body) ) {
-                my_body = new( p->zombie_space.begin() ) Body(*my_body,split());
-                p->has_right_zombie = true;
-            } 
+        my_partition.check_being_stolen( *this );
+        if( my_context==2 ) { // right child
+            finish_type* parent_ptr = static_cast<finish_type*>(parent());
+            if( !itt_load_word_with_acquire(parent_ptr->my_body) ) { // TODO: replace by is_stolen_task() or by parent_ptr->ref_count() == 2???
+                my_body = new( parent_ptr->zombie_space.begin() ) Body(*my_body,split());
+                parent_ptr->has_right_zombie = true;
+            }
+        } else __TBB_ASSERT(my_context==0,0);// because left leaf spawns right leafs without recycling
+        my_partition.execute(*this, my_range);
+        if( my_context==1 ) {
+            finish_type* parent_ptr = static_cast<finish_type*>(parent());
+            __TBB_ASSERT(my_body!=parent_ptr->zombie_space.begin(),0);
+            itt_store_word_with_release(parent_ptr->my_body, my_body );
         }
-        if( !my_range.is_divisible() || my_partition.should_execute_range(*this) ) {
-            (*my_body)( my_range );
-            if( my_context==1 ) 
-                parallel_reduce_store_body(static_cast<finish_type*>(parent())->my_body, my_body );
-            return my_partition.continue_after_execute_range();
+        return NULL;
+    }
+
+#if TBB_PREVIEW_DETERMINISTIC_REDUCE
+    //! Task type use to combine the partial results of parallel_deterministic_reduce.
+    /** @ingroup algorithms */
+    template<typename Body>
+    class finish_deterministic_reduce: public task {
+        Body &my_left_body;
+        Body my_right_body;
+
+        finish_deterministic_reduce( Body &body ) :
+            my_left_body( body ),
+            my_right_body( body, split() )
+        {
+        }
+        task* execute() {
+            my_left_body.join( my_right_body );
+            return NULL;
+        }
+        template<typename Range,typename Body_>
+        friend class start_deterministic_reduce;
+    };
+
+    //! Task type used to split the work of parallel_deterministic_reduce.
+    /** @ingroup algorithms */
+    template<typename Range, typename Body>
+    class start_deterministic_reduce: public task {
+        typedef finish_deterministic_reduce<Body> finish_type;
+        Body &my_body;
+        Range my_range;
+        /*override*/ task* execute();
+
+        //! Constructor used for root task
+        start_deterministic_reduce( const Range& range, Body& body ) :
+            my_body( body ),
+            my_range( range )
+        {
+        }
+        //! Splitting constructor used to generate children.
+        /** parent_ becomes left child.  Newly constructed object is right child. */
+        start_deterministic_reduce( start_deterministic_reduce& parent_, finish_type& c ) :
+            my_body( c.my_right_body ),
+            my_range( parent_.my_range, split() )
+        {
+        }
+
+public:
+        static void run( const Range& range, Body& body ) {
+            if( !range.empty() ) {
+#if !__TBB_TASK_GROUP_CONTEXT || TBB_JOIN_OUTER_TASK_GROUP
+                task::spawn_root_and_wait( *new(task::allocate_root()) start_deterministic_reduce(range,&body) );
+#else
+                // Bound context prevents exceptions from body to affect nesting or sibling algorithms,
+                // and allows users to handle exceptions safely by wrapping parallel_for in the try-block.
+                task_group_context context;
+                task::spawn_root_and_wait( *new(task::allocate_root(context)) start_deterministic_reduce(range,body) );
+#endif /* __TBB_TASK_GROUP_CONTEXT && !TBB_JOIN_OUTER_TASK_GROUP */
+            }
+        }
+#if __TBB_TASK_GROUP_CONTEXT
+        static void run( const Range& range, Body& body, task_group_context& context ) {
+            if( !range.empty() ) 
+                task::spawn_root_and_wait( *new(task::allocate_root(context)) start_deterministic_reduce(range,body) );
+        }
+#endif /* __TBB_TASK_GROUP_CONTEXT */
+    };
+
+    template<typename Range, typename Body>
+    task* start_deterministic_reduce<Range,Body>::execute() {
+        if( !my_range.is_divisible() ) {
+            my_body( my_range );
+            return NULL;
         } else {
-            finish_type& c = *new( allocate_continuation()) finish_type(my_context);
+            finish_type& c = *new( allocate_continuation() ) finish_type( my_body );
             recycle_as_child_of(c);
-            c.set_ref_count(2);    
-            bool delay = my_partition.decide_whether_to_delay();
-            start_reduce& b = *new( c.allocate_child() ) start_reduce(*this,split());
-            my_partition.spawn_or_delay(delay,b);
+            c.set_ref_count(2);
+            start_deterministic_reduce& b = *new( c.allocate_child() ) start_deterministic_reduce( *this, c );
+            task::spawn(b);
             return this;
         }
-    } 
+    }
+#endif /* TBB_PREVIEW_DETERMINISTIC_REDUCE */
+} // namespace internal
+//! @endcond
+} //namespace interfaceX
 
+//! @cond INTERNAL
+namespace internal {
+    using interface6::internal::start_reduce;
+#if TBB_PREVIEW_DETERMINISTIC_REDUCE
+    using interface6::internal::start_deterministic_reduce;
+#endif
     //! Auxiliary class for parallel_reduce; for internal use only.
     /** The adaptor class that implements \ref parallel_reduce_body_req "parallel_reduce Body"
         using given \ref parallel_reduce_lambda_req "anonymous function objects".
@@ -379,6 +461,50 @@ Value parallel_reduce( const Range& range, const Value& identity, const RealBody
     return body.result();
 }
 #endif /* __TBB_TASK_GROUP_CONTEXT */
+
+#if TBB_PREVIEW_DETERMINISTIC_REDUCE
+//! Parallel iteration with deterministic reduction and default partitioner.
+/** @ingroup algorithms **/
+template<typename Range, typename Body>
+void parallel_deterministic_reduce( const Range& range, Body& body ) {
+    internal::start_deterministic_reduce<Range,Body>::run( range, body );
+}
+
+#if __TBB_TASK_GROUP_CONTEXT
+//! Parallel iteration with deterministic reduction, simple partitioner and user-supplied context.
+/** @ingroup algorithms **/
+template<typename Range, typename Body>
+void parallel_deterministic_reduce( const Range& range, Body& body, task_group_context& context ) {
+    internal::start_deterministic_reduce<Range,Body>::run( range, body, context );
+}
+#endif /* __TBB_TASK_GROUP_CONTEXT */
+
+/** parallel_reduce overloads that work with anonymous function objects
+    (see also \ref parallel_reduce_lambda_req "requirements on parallel_reduce anonymous function objects"). **/
+
+//! Parallel iteration with deterministic reduction and default partitioner.
+/** @ingroup algorithms **/
+template<typename Range, typename Value, typename RealBody, typename Reduction>
+Value parallel_deterministic_reduce( const Range& range, const Value& identity, const RealBody& real_body, const Reduction& reduction ) {
+    internal::lambda_reduce_body<Range,Value,RealBody,Reduction> body(identity, real_body, reduction);
+    internal::start_deterministic_reduce<Range,internal::lambda_reduce_body<Range,Value,RealBody,Reduction> >
+                          ::run(range, body);
+    return body.result();
+}
+
+#if __TBB_TASK_GROUP_CONTEXT
+//! Parallel iteration with deterministic reduction, simple partitioner and user-supplied context.
+/** @ingroup algorithms **/
+template<typename Range, typename Value, typename RealBody, typename Reduction>
+Value parallel_deterministic_reduce( const Range& range, const Value& identity, const RealBody& real_body, const Reduction& reduction,
+                       task_group_context& context ) {
+    internal::lambda_reduce_body<Range,Value,RealBody,Reduction> body(identity, real_body, reduction);
+    internal::start_deterministic_reduce<Range,internal::lambda_reduce_body<Range,Value,RealBody,Reduction> >
+                          ::run( range, body, context );
+    return body.result();
+}
+#endif /* __TBB_TASK_GROUP_CONTEXT */
+#endif /* TBB_PREVIEW_DETERMINISTIC_REDUCE */
 //@}
 
 } // namespace tbb
