@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2010 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -27,19 +27,22 @@
 */
 
 // configuration:
+#define TBB_USE_THREADING_TOOLS 0
 
 //! enable/disable std::map tests
-#define STDTABLE 1
+#define STDTABLE 0
 
 //! enable/disable old implementation tests (correct include file also)
 #define OLDTABLE 0
-#define OLDTABLEHEADER "tbb/concurrent_hash_map-4078.h"//-4329
+#define OLDTABLEHEADER "tbb/concurrent_hash_map-5468.h"//-4329
 
 //! enable/disable experimental implementation tests (correct include file also)
-
-#define TESTTABLE 0
+#define TESTTABLE 1
 #define TESTTABLEHEADER "tbb/concurrent_unordered_map.h"
 
+//! avoid erase()
+#define TEST_ERASE 0
+
 //////////////////////////////////////////////////////////////////////////////////
 
 #include <cstdlib>
@@ -57,6 +60,7 @@
 #include "tbb/spin_rw_mutex.h"
 #include "tbb/aligned_space.h"
 #include "tbb/atomic.h"
+#include "tbb/internal/_concurrent_unordered_impl.h"
 // for test
 #include "tbb/spin_mutex.h"
 #include "time_framework.h"
@@ -74,43 +78,44 @@ struct IntHashCompare {
 
 namespace version_current {
     namespace tbb { using namespace ::tbb; namespace internal { using namespace ::tbb::internal; } }
+    namespace tbb { namespace interface5 { using namespace ::tbb::interface5; namespace internal { using namespace ::tbb::interface5::internal; } } }
     #include "tbb/concurrent_hash_map.h"
 }
-typedef version_current::tbb::concurrent_hash_map<int,int,IntHashCompare> IntTable;
+typedef version_current::tbb::concurrent_hash_map<int,int> IntTable;
 
 #if OLDTABLE
 #undef __TBB_concurrent_hash_map_H
 namespace version_base {
     namespace tbb { using namespace ::tbb; namespace internal { using namespace ::tbb::internal; } }
+    namespace tbb { namespace interface5 { using namespace ::tbb::interface5; namespace internal { using namespace ::tbb::interface5::internal; } } }
     #include OLDTABLEHEADER
 }
-typedef version_base::tbb::concurrent_hash_map<int,int,IntHashCompare> OldTable;
+typedef version_base::tbb::concurrent_hash_map<int,int> OldTable;
 #endif
 
 #if TESTTABLE
 #undef __TBB_concurrent_hash_map_H
 namespace version_new {
     namespace tbb { using namespace ::tbb; namespace internal { using namespace ::tbb::internal; } }
+    namespace tbb { namespace interface5 { using namespace ::tbb::interface5; namespace internal { using namespace ::tbb::interface5::internal; } } }
     #include TESTTABLEHEADER
 }
-typedef version_new::tbb::concurrent_unordered_map<int,int,IntHashCompare,IntHashCompare> TestTable;
+typedef version_new::tbb::concurrent_unordered_map<int,int> TestTable;
 #define TESTTABLE 1
 #endif
 
 ///////////////////////////////////////
 
 static const char *map_testnames[] = {
-    "1.insert", "2.count(w/rehash)", "3.find/wr", "4.erase"
+    "1.insert", "2.count1st", "3.count2nd", "4.insert existing", "5.erase"
 };
 
 template<typename TableType>
 struct TestTBBMap : TesterBase {
-    typedef typename TableType::accessor accessor;
-    typedef typename TableType::const_accessor const_accessor;
     TableType Table;
     int n_items;
 
-    TestTBBMap() : TesterBase(4) {}
+    TestTBBMap() : TesterBase(4+TEST_ERASE), Table(MaxThread*4) {}
     void init() { n_items = value/threads_count; }
 
     std::string get_name(int testn) {
@@ -122,9 +127,7 @@ struct TestTBBMap : TesterBase {
         switch(test) {
           case 0: // fill
             for(int i = t*n_items, e = (t+1)*n_items; i < e; i++) {
-                accessor a;
-                Table.insert( a, i );
-                a->second = 0;
+                Table.insert( std::make_pair(i,i) );
             }
             break;
           case 1: // work1
@@ -135,16 +138,20 @@ struct TestTBBMap : TesterBase {
             break;
           case 2: // work2
             for(int i = t*n_items, e = (t+1)*n_items; i < e; i++) {
-                accessor a;
-                Table.find( a, i );
-                ASSERT( !a->second, "A key should be incremented only once");
-                a->second += 1;
+                Table.count( i );
             }
             break;
-          case 3: // clean
+          case 3: // work3
+            for(int i = t*n_items, e = (t+1)*n_items; i < e; i++) {
+                Table.insert( std::make_pair(i,i) );
+            }
+            break;
+#if TEST_ERASE
+          case 4: // clean
             for(int i = t*n_items, e = (t+1)*n_items; i < e; i++) {
                 ASSERT( Table.erase( i ), NULL);
             }
+#endif
         }
         return 0;
     }
@@ -156,7 +163,7 @@ struct TestSTLMap : TesterBase {
     M mutex;
 
     int n_items;
-    TestSTLMap() : TesterBase(4) {}
+    TestSTLMap() : TesterBase(4+TEST_ERASE) {}
     void init() { n_items = value/threads_count; }
 
     std::string get_name(int testn) {
@@ -182,10 +189,16 @@ struct TestSTLMap : TesterBase {
           case 2: // work2
             for(int i = t*n_items, e = (t+1)*n_items; i < e; i++) {
                 typename M::scoped_lock with(mutex);
-                Table[i] += 1;
+                Table.count(i);
             }
             break;
-          case 3: // clean
+          case 3: // work3
+            for(int i = t*n_items, e = (t+1)*n_items; i < e; i++) {
+                typename M::scoped_lock with(mutex);
+                Table.insert(std::make_pair(i,i));
+            }
+            break;
+          case 4: // clean
             for(int i = t*n_items, e = (t+1)*n_items; i < e; i++) {
                 typename M::scoped_lock with(mutex);
                 Table.erase(i);
@@ -196,7 +209,6 @@ struct TestSTLMap : TesterBase {
 };
 
 class fake_mutex {
-    int a;
 public:
     class scoped_lock {
         fake_mutex *p;
@@ -204,7 +216,7 @@ public:
     public:
         scoped_lock() {}
         scoped_lock( fake_mutex &m ) { p = &m; }
-        ~scoped_lock() { p->a = 0; }
+        ~scoped_lock() { }
         void acquire( fake_mutex &m ) { p = &m; }
         void release() { }
     };
@@ -228,108 +240,7 @@ public:
 #endif
         end );
         //stat->Print(StatisticsCollector::Stdout);
-        if(value >= 2097152) stat->Print(StatisticsCollector::HTMLFile);
-    }
-};
-
-/////////////////////////////////////////////////////////////////////////////////////////
-template<typename TableType>
-struct TestHashMapFind : TesterBase {
-    typedef typename TableType::accessor accessor;
-    typedef typename TableType::const_accessor const_accessor;
-    TableType Table;
-    int n_items;
-
-    std::string get_name(int testn) {
-        return std::string(!testn?"find":"insert");
-    }
-
-    TestHashMapFind() : TesterBase(2) {}
-    void init() {
-        n_items = value/threads_count;
-        for(int i = 0; i < value; i++) {
-            accessor a; Table.insert( a, i );
-        }
-    }
-
-    double test(int test, int t)
-    {
-        switch(test) {
-          case 0: // find
-            for(int i = t*n_items, e = (t+1)*n_items; i < e; i++) {
-                accessor a;
-                Table.find( a, i );
-                a->second = i;
-            }
-            break;
-          case 1: // insert
-            for(int i = t*n_items, e = (t+1)*n_items; i < e; i++) {
-                accessor a;
-                Table.insert( a, i );
-                a->second = -i;
-            }
-            break;
-        }
-        return 0;
-    }
-};
-
-const int test2_size = 65536;
-int Data[test2_size];
-
-template<typename TableType>
-struct TestHashCountStrings : TesterBase {
-    typedef typename TableType::accessor accessor;
-    typedef typename TableType::const_accessor const_accessor;
-    TableType Table;
-    int n_items;
-
-    std::string get_name(int testn) {
-        return !testn?"insert":"find";
-    }
-
-    TestHashCountStrings() : TesterBase(2) {}
-    void init() {
-        n_items = value/threads_count;
-    }
-
-    double test(int testn, int t)
-    {
-        if(!testn) {
-            for(int i = t*n_items, e = (t+1)*n_items; i < e; i++) {
-                accessor a; Table.insert( a, Data[i] );
-            }
-        } else { // 
-            for(int i = t*n_items, e = (t+1)*n_items; i < e; i++) {
-                accessor a; Table.find( a, Data[i] );
-            }
-        }
-        return 0;
-    }
-};
-
-class test_hash_map_find : public TestProcessor {
-public:
-    test_hash_map_find() : TestProcessor("test_hash_map_find") {}
-    void factory(int value, int threads) {
-        if(Verbose) printf("Processing with %d threads: %d...\n", threads, value);
-        process( value, threads,
-#if OLDTABLE
-            run("Filled old::hashmap", new NanosecPerValue<TestHashMapFind<OldTable> >() ),
-#endif
-            run("Filled tbb::hashmap", new NanosecPerValue<TestHashMapFind<IntTable> >() ),
-#if TESTTABLE
-            run("Filled new::hashmap", new NanosecPerValue<TestHashMapFind<TestTable> >() ),
-#endif
-#if OLDTABLE
-            run("CountStr old::hashmap", new TimeTest<TestHashCountStrings<OldTable> >() ),
-#endif
-            run("CountStr tbb::hashmap", new TimeTest<TestHashCountStrings<IntTable> >() ),
-#if TESTTABLE
-            run("CountStr new::hashmap", new TimeTest<TestHashCountStrings<TestTable> >() ),
-#endif
-        end );
-        //stat->Print(StatisticsCollector::HTMLFile);
+        //if(value >= 2097152) stat->Print(StatisticsCollector::HTMLFile);
     }
 };
 
@@ -343,18 +254,9 @@ int main(int argc, char* argv[]) {
 
     ASSERT(tbb_allocator<int>::allocator_type() == tbb_allocator<int>::scalable, "expecting scalable allocator library to be loaded. Please build it by:\n\t\tmake tbbmalloc");
 
-    {
-        test_hash_map_find test_find; int o = test2_size;
-        for(int i = 0; i < o; i++)
-            Data[i] = i%60;
-        for( int t=MinThread; t <= MaxThread; t++)
-            test_find.factory(o, t);
-        test_find.report.SetTitle("Nanoseconds per operation of finding operation (Mode) for %d items", o);
-        test_find.report.Print(StatisticsCollector::HTMLFile|StatisticsCollector::ExcelXML);
-    }
     {
         test_hash_map the_test;
-        for( int t=MinThread; t <= MaxThread; t*=2)
+        for( int t=MinThread; t <= MaxThread; t++)
             for( int o=/*2048*/(1<<8)*8; o<2200000; o*=2 )
                 the_test.factory(o, t);
         the_test.report.SetTitle("Nanoseconds per operation of (Mode) for N items in container (Name)");
