@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2010 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -58,7 +58,7 @@ enum state_t_flags {
 
 const unsigned char RELEASED = 0;
 const unsigned char ACQUIRED = 1;
- 
+
 template<typename T>
 inline atomic<T>& as_atomic( T& t ) {
     return *(atomic<T>*)&t;
@@ -66,7 +66,7 @@ inline atomic<T>& as_atomic( T& t ) {
 
 inline bool queuing_rw_mutex::scoped_lock::try_acquire_internal_lock()
 {
-    return as_atomic(internal_lock).compare_and_swap<tbb::acquire>(ACQUIRED,RELEASED) == RELEASED;
+    return as_atomic(my_internal_lock).compare_and_swap<tbb::acquire>(ACQUIRED,RELEASED) == RELEASED;
 }
 
 inline void queuing_rw_mutex::scoped_lock::acquire_internal_lock()
@@ -80,12 +80,12 @@ inline void queuing_rw_mutex::scoped_lock::acquire_internal_lock()
 
 inline void queuing_rw_mutex::scoped_lock::release_internal_lock()
 {
-    __TBB_store_with_release(internal_lock,RELEASED);
+    __TBB_store_with_release(my_internal_lock,RELEASED);
 }
 
 inline void queuing_rw_mutex::scoped_lock::wait_for_release_of_internal_lock()
 {
-    spin_wait_until_eq(internal_lock, RELEASED);
+    spin_wait_until_eq(my_internal_lock, RELEASED);
 }
 
 inline void queuing_rw_mutex::scoped_lock::unblock_or_wait_on_internal_lock( uintptr_t flag ) {
@@ -105,7 +105,7 @@ inline void queuing_rw_mutex::scoped_lock::unblock_or_wait_on_internal_lock( uin
 template<typename T>
 class tricky_atomic_pointer: no_copy {
 public:
-    typedef typename atomic_rep<sizeof(T*)>::word word;
+    typedef typename atomic_selector<sizeof(T*)>::word word;
 
     template<memory_semantics M>
     static T* fetch_and_add( T* volatile * location, word addend ) {
@@ -145,8 +145,8 @@ typedef tricky_atomic_pointer<queuing_rw_mutex::scoped_lock> tricky_pointer;
 static const tricky_pointer::word FLAG = 0x1;
 
 inline
-uintptr_t get_flag( queuing_rw_mutex::scoped_lock* ptr ) { 
-    return uintptr_t(tricky_pointer(ptr)&FLAG);
+uintptr_t get_flag( queuing_rw_mutex::scoped_lock* ptr ) {
+    return uintptr_t(ptr) & FLAG;
 }
 
 //------------------------------------------------------------------------
@@ -155,29 +155,31 @@ uintptr_t get_flag( queuing_rw_mutex::scoped_lock* ptr ) {
 
 void queuing_rw_mutex::scoped_lock::acquire( queuing_rw_mutex& m, bool write )
 {
-    __TBB_ASSERT( !this->mutex, "scoped_lock is already holding a mutex");
+    __TBB_ASSERT( !my_mutex, "scoped_lock is already holding a mutex");
 
     // Must set all fields before the fetch_and_store, because once the
     // fetch_and_store executes, *this becomes accessible to other threads.
-    mutex = &m;
-    prev  = NULL;
-    next  = NULL;
-    going = 0;
-    state = state_t(write ? STATE_WRITER : STATE_READER);
-    internal_lock = RELEASED;
+    my_mutex = &m;
+    __TBB_store_relaxed(my_prev , (scoped_lock*)0);
+    __TBB_store_relaxed(my_next , (scoped_lock*)0);
+    __TBB_store_relaxed(my_going, 0);
+    my_state = state_t(write ? STATE_WRITER : STATE_READER);
+    my_internal_lock = RELEASED;
 
     queuing_rw_mutex::scoped_lock* pred = m.q_tail.fetch_and_store<tbb::release>(this);
 
     if( write ) {       // Acquiring for write
 
         if( pred ) {
-            ITT_NOTIFY(sync_prepare, mutex);
+            ITT_NOTIFY(sync_prepare, my_mutex);
             pred = tricky_pointer(pred) & ~FLAG;
-            __TBB_ASSERT( !( tricky_pointer(pred) & FLAG ), "use of corrupted pointer!" );
-            __TBB_ASSERT( !pred->next, "the predecessor has another successor!");
-            // ensure release semantics on IPF
-           __TBB_store_with_release(pred->next,this);
-            spin_wait_until_eq(going, 1);
+            __TBB_ASSERT( !( uintptr_t(pred) & FLAG ), "use of corrupted pointer!" );
+#if TBB_USE_ASSERT
+            __TBB_control_consistency_helper(); // on "m.q_tail"
+            __TBB_ASSERT( !__TBB_load_relaxed(pred->my_next), "the predecessor has another successor!");
+#endif
+           __TBB_store_with_release(pred->my_next,this);
+            spin_wait_until_eq(my_going, 1);
         }
 
     } else {            // Acquiring for read
@@ -186,135 +188,134 @@ void queuing_rw_mutex::scoped_lock::acquire( queuing_rw_mutex& m, bool write )
 #endif
         if( pred ) {
             unsigned short pred_state;
-            __TBB_ASSERT( !this->prev, "the predecessor is already set" );
-            if( tricky_pointer(pred)&FLAG ) {
+            __TBB_ASSERT( !__TBB_load_relaxed(my_prev), "the predecessor is already set" );
+            if( uintptr_t(pred) & FLAG ) {
                 /* this is only possible if pred is an upgrading reader and it signals us to wait */
                 pred_state = STATE_UPGRADE_WAITING;
                 pred = tricky_pointer(pred) & ~FLAG;
             } else {
-                // Load pred->state now, because once pred->next becomes
+                // Load pred->my_state now, because once pred->my_next becomes
                 // non-NULL, we must assume that *pred might be destroyed.
-                pred_state = pred->state.compare_and_swap<tbb::acquire>(STATE_READER_UNBLOCKNEXT, STATE_READER);
+                pred_state = pred->my_state.compare_and_swap<tbb::acquire>(STATE_READER_UNBLOCKNEXT, STATE_READER);
             }
-            this->prev = pred;
-            __TBB_ASSERT( !( tricky_pointer(pred) & FLAG ), "use of corrupted pointer!" );
-            __TBB_ASSERT( !pred->next, "the predecessor has another successor!");
-            // ensure release semantics on IPF
-           __TBB_store_with_release(pred->next,this);
+            __TBB_store_relaxed(my_prev, pred);
+            __TBB_ASSERT( !( uintptr_t(pred) & FLAG ), "use of corrupted pointer!" );
+#if TBB_USE_ASSERT
+            __TBB_control_consistency_helper(); // on "m.q_tail"
+            __TBB_ASSERT( !__TBB_load_relaxed(pred->my_next), "the predecessor has another successor!");
+#endif
+           __TBB_store_with_release(pred->my_next,this);
             if( pred_state != STATE_ACTIVEREADER ) {
 #if DO_ITT_NOTIFY
                 sync_prepare_done = true;
-                ITT_NOTIFY(sync_prepare, mutex);
+                ITT_NOTIFY(sync_prepare, my_mutex);
 #endif
-                spin_wait_until_eq(going, 1);
+                spin_wait_until_eq(my_going, 1);
             }
         }
-        unsigned short old_state = state.compare_and_swap<tbb::acquire>(STATE_ACTIVEREADER, STATE_READER);
+        unsigned short old_state = my_state.compare_and_swap<tbb::acquire>(STATE_ACTIVEREADER, STATE_READER);
         if( old_state!=STATE_READER ) {
 #if DO_ITT_NOTIFY
             if( !sync_prepare_done )
-                ITT_NOTIFY(sync_prepare, mutex);
+                ITT_NOTIFY(sync_prepare, my_mutex);
 #endif
             // Failed to become active reader -> need to unblock the next waiting reader first
-            __TBB_ASSERT( state==STATE_READER_UNBLOCKNEXT, "unexpected state" );
-            spin_wait_while_eq(next, (scoped_lock*)NULL);
-            /* state should be changed before unblocking the next otherwise it might finish
+            __TBB_ASSERT( my_state==STATE_READER_UNBLOCKNEXT, "unexpected state" );
+            spin_wait_while_eq(my_next, (scoped_lock*)NULL);
+            /* my_state should be changed before unblocking the next otherwise it might finish
                and another thread can get our old state and left blocked */
-            state = STATE_ACTIVEREADER;
-            // ensure release semantics on IPF
-           __TBB_store_with_release(next->going,1);
+            my_state = STATE_ACTIVEREADER;
+           __TBB_store_with_release(my_next->my_going,1);
         }
     }
 
-    ITT_NOTIFY(sync_acquired, mutex);
+    ITT_NOTIFY(sync_acquired, my_mutex);
 
     // Force acquire so that user's critical section receives correct values
     // from processor that was previously in the user's critical section.
-    __TBB_load_with_acquire(going);
+    __TBB_load_with_acquire(my_going);
 }
 
 bool queuing_rw_mutex::scoped_lock::try_acquire( queuing_rw_mutex& m, bool write )
 {
-    __TBB_ASSERT( !this->mutex, "scoped_lock is already holding a mutex");
+    __TBB_ASSERT( !my_mutex, "scoped_lock is already holding a mutex");
+
+    if( load<relaxed>(m.q_tail) )
+        return false; // Someone already took the lock
 
     // Must set all fields before the fetch_and_store, because once the
     // fetch_and_store executes, *this becomes accessible to other threads.
-    prev  = NULL;
-    next  = NULL;
-    going = 0;
-    state = state_t(write ? STATE_WRITER : STATE_ACTIVEREADER);
-    internal_lock = RELEASED;
+    __TBB_store_relaxed(my_prev, (scoped_lock*)0);
+    __TBB_store_relaxed(my_next, (scoped_lock*)0);
+    __TBB_store_relaxed(my_going, 0); // TODO: remove dead assignment?
+    my_state = state_t(write ? STATE_WRITER : STATE_ACTIVEREADER);
+    my_internal_lock = RELEASED;
 
-    if( m.q_tail ) return false;
     // The CAS must have release semantics, because we are
     // "sending" the fields initialized above to other processors.
-    queuing_rw_mutex::scoped_lock* pred = m.q_tail.compare_and_swap<tbb::release>(this, NULL);
-
+    if( m.q_tail.compare_and_swap<tbb::release>(this, NULL) )
+        return false; // Someone already took the lock
     // Force acquire so that user's critical section receives correct values
     // from processor that was previously in the user's critical section.
     // try_acquire should always have acquire semantic, even if failed.
-    __TBB_load_with_acquire(going);
-
-    if( !pred ) {
-        mutex = &m;
-        ITT_NOTIFY(sync_acquired, mutex);
-        return true;
-    } else return false;
-
+    __TBB_load_with_acquire(my_going);
+    my_mutex = &m;
+    ITT_NOTIFY(sync_acquired, my_mutex);
+    return true;
 }
 
 void queuing_rw_mutex::scoped_lock::release( )
 {
-    __TBB_ASSERT(this->mutex!=NULL, "no lock acquired");
+    __TBB_ASSERT(my_mutex!=NULL, "no lock acquired");
 
-    ITT_NOTIFY(sync_releasing, mutex);
+    ITT_NOTIFY(sync_releasing, my_mutex);
 
-    if( state == STATE_WRITER ) { // Acquired for write
+    if( my_state == STATE_WRITER ) { // Acquired for write
 
-        // The logic below is the same as "writerUnlock", but restructured to remove "return" in the middle of routine.
-        // In the statement below, acquire semantics of reading 'next' is required
-        // so that following operations with fields of 'next' are safe.
-        scoped_lock* n = __TBB_load_with_acquire(next);
+        // The logic below is the same as "writerUnlock", but elides
+        // "return" from the middle of the routine.
+        // In the statement below, acquire semantics of reading my_next is required
+        // so that following operations with fields of my_next are safe.
+        scoped_lock* n = __TBB_load_with_acquire(my_next);
         if( !n ) {
-            if( this == mutex->q_tail.compare_and_swap<tbb::release>(NULL, this) ) {
+            if( this == my_mutex->q_tail.compare_and_swap<tbb::release>(NULL, this) ) {
                 // this was the only item in the queue, and the queue is now empty.
                 goto done;
             }
-            spin_wait_while_eq( next, (scoped_lock*)NULL );
-            n = next;
+            spin_wait_while_eq( my_next, (scoped_lock*)NULL );
+            n = __TBB_load_with_acquire(my_next);
         }
-        n->going = 2; // protect next queue node from being destroyed too early
-        if( n->state==STATE_UPGRADE_WAITING ) {
+        __TBB_store_relaxed(n->my_going, 2); // protect next queue node from being destroyed too early
+        if( n->my_state==STATE_UPGRADE_WAITING ) {
             // the next waiting for upgrade means this writer was upgraded before.
             acquire_internal_lock();
-            queuing_rw_mutex::scoped_lock* tmp = tricky_pointer::fetch_and_store<tbb::release>(&(n->prev), NULL);
-            n->state = STATE_UPGRADE_LOSER;
-            __TBB_store_with_release(n->going,1);
+            queuing_rw_mutex::scoped_lock* tmp = tricky_pointer::fetch_and_store<tbb::release>(&(n->my_prev), NULL);
+            n->my_state = STATE_UPGRADE_LOSER;
+            __TBB_store_with_release(n->my_going,1);
             unblock_or_wait_on_internal_lock(get_flag(tmp));
         } else {
-            __TBB_ASSERT( state & (STATE_COMBINED_WAITINGREADER | STATE_WRITER), "unexpected state" );
-            __TBB_ASSERT( !( tricky_pointer(n->prev) & FLAG ), "use of corrupted pointer!" );
-            n->prev = NULL;
-            // ensure release semantics on IPF
-            __TBB_store_with_release(n->going,1);
+            __TBB_ASSERT( my_state & (STATE_COMBINED_WAITINGREADER | STATE_WRITER), "unexpected state" );
+            __TBB_ASSERT( !( uintptr_t(__TBB_load_relaxed(n->my_prev)) & FLAG ), "use of corrupted pointer!" );
+            __TBB_store_relaxed(n->my_prev, (scoped_lock*)0);
+            __TBB_store_with_release(n->my_going,1);
         }
 
     } else { // Acquired for read
 
         queuing_rw_mutex::scoped_lock *tmp = NULL;
 retry:
-        // Addition to the original paper: Mark this->prev as in use
-        queuing_rw_mutex::scoped_lock *pred = tricky_pointer::fetch_and_add<tbb::acquire>(&(this->prev), FLAG);
+        // Addition to the original paper: Mark my_prev as in use
+        queuing_rw_mutex::scoped_lock *pred = tricky_pointer::fetch_and_add<tbb::acquire>(&my_prev, FLAG);
 
         if( pred ) {
             if( !(pred->try_acquire_internal_lock()) )
             {
                 // Failed to acquire the lock on pred. The predecessor either unlinks or upgrades.
                 // In the second case, it could or could not know my "in use" flag - need to check
-                tmp = tricky_pointer::compare_and_swap<tbb::release>(&(this->prev), pred, tricky_pointer(pred)|FLAG );
-                if( !(tricky_pointer(tmp)&FLAG) ) {
-                    // Wait for the predecessor to change this->prev (e.g. during unlink)
-                    spin_wait_while_eq( this->prev, tricky_pointer(pred)|FLAG );
+                tmp = tricky_pointer::compare_and_swap<tbb::release>(&my_prev, pred, tricky_pointer(pred) | FLAG );
+                if( !(uintptr_t(tmp) & FLAG) ) {
+                    // Wait for the predecessor to change my_prev (e.g. during unlink)
+                    spin_wait_while_eq( my_prev, tricky_pointer(pred)|FLAG );
                     // Now owner of pred is waiting for _us_ to release its lock
                     pred->release_internal_lock();
                 }
@@ -323,178 +324,177 @@ retry:
                 tmp = NULL;
                 goto retry;
             }
-            __TBB_ASSERT(pred && pred->internal_lock==ACQUIRED, "predecessor's lock is not acquired");
-            this->prev = pred;
+            __TBB_ASSERT(pred && pred->my_internal_lock==ACQUIRED, "predecessor's lock is not acquired");
+            __TBB_store_relaxed(my_prev, pred);
             acquire_internal_lock();
 
-            __TBB_store_with_release(pred->next,reinterpret_cast<scoped_lock *>(NULL));
+            __TBB_store_with_release(pred->my_next,reinterpret_cast<scoped_lock *>(NULL));
 
-            if( !next && this != mutex->q_tail.compare_and_swap<tbb::release>(pred, this) ) {
-                spin_wait_while_eq( next, (void*)NULL );
+            if( !__TBB_load_relaxed(my_next) && this != my_mutex->q_tail.compare_and_swap<tbb::release>(pred, this) ) {
+                spin_wait_while_eq( my_next, (void*)NULL );
             }
-            __TBB_ASSERT( !get_flag(next), "use of corrupted pointer" );
+            __TBB_ASSERT( !get_flag(__TBB_load_relaxed(my_next)), "use of corrupted pointer" );
 
-            // ensure acquire semantics of reading 'next'
-            if( __TBB_load_with_acquire(next) ) { // I->next != nil
+            // ensure acquire semantics of reading 'my_next'
+            if( scoped_lock *const l_next = __TBB_load_with_acquire(my_next) ) { // I->next != nil, TODO: rename to n after clearing up and adapting the n in the comment two lines below
                 // Equivalent to I->next->prev = I->prev but protected against (prev[n]&FLAG)!=0
-                tmp = tricky_pointer::fetch_and_store<tbb::release>(&(next->prev), pred);
+                tmp = tricky_pointer::fetch_and_store<tbb::release>(&(l_next->my_prev), pred);
                 // I->prev->next = I->next;
-                __TBB_ASSERT(this->prev==pred, NULL);
-                __TBB_store_with_release(pred->next,next);
+                __TBB_ASSERT(__TBB_load_relaxed(my_prev)==pred, NULL);
+                __TBB_store_with_release(pred->my_next, my_next);
             }
             // Safe to release in the order opposite to acquiring which makes the code simplier
             pred->release_internal_lock();
 
         } else { // No predecessor when we looked
             acquire_internal_lock();  // "exclusiveLock(&I->EL)"
-            // ensure acquire semantics of reading 'next'
-            scoped_lock* n = __TBB_load_with_acquire(next);
+            scoped_lock* n = __TBB_load_with_acquire(my_next);
             if( !n ) {
-                if( this != mutex->q_tail.compare_and_swap<tbb::release>(NULL, this) ) {
-                    spin_wait_while_eq( next, (scoped_lock*)NULL );
-                    n = next;
+                if( this != my_mutex->q_tail.compare_and_swap<tbb::release>(NULL, this) ) {
+                    spin_wait_while_eq( my_next, (scoped_lock*)NULL );
+                    n = __TBB_load_relaxed(my_next);
                 } else {
                     goto unlock_self;
                 }
             }
-            n->going = 2; // protect next queue node from being destroyed too early
-            tmp = tricky_pointer::fetch_and_store<tbb::release>(&(n->prev), NULL);
-            // ensure release semantics on IPF
-            __TBB_store_with_release(n->going,1);
+            __TBB_store_relaxed(n->my_going, 2); // protect next queue node from being destroyed too early
+            tmp = tricky_pointer::fetch_and_store<tbb::release>(&(n->my_prev), NULL);
+            __TBB_store_with_release(n->my_going,1);
         }
 unlock_self:
         unblock_or_wait_on_internal_lock(get_flag(tmp));
     }
 done:
-    spin_wait_while_eq( going, 2 );
+    spin_wait_while_eq( my_going, 2 );
 
     initialize();
 }
 
 bool queuing_rw_mutex::scoped_lock::downgrade_to_reader()
 {
-    __TBB_ASSERT( state==STATE_WRITER, "no sense to downgrade a reader" );
+    __TBB_ASSERT( my_state==STATE_WRITER, "no sense to downgrade a reader" );
 
-    ITT_NOTIFY(sync_releasing, mutex);
+    ITT_NOTIFY(sync_releasing, my_mutex);
 
-    // ensure acquire semantics of reading 'next'
-    if( ! __TBB_load_with_acquire(next) ) {
-        state = STATE_READER;
-        if( this==mutex->q_tail ) {
-            unsigned short old_state = state.compare_and_swap<tbb::release>(STATE_ACTIVEREADER, STATE_READER);
+    if( ! __TBB_load_with_acquire(my_next) ) {
+        my_state = STATE_READER;
+        if( this==my_mutex->q_tail ) {
+            unsigned short old_state = my_state.compare_and_swap<tbb::release>(STATE_ACTIVEREADER, STATE_READER);
             if( old_state==STATE_READER ) {
-                goto downgrade_done;
+                // Downgrade completed
+                return true;
             }
         }
         /* wait for the next to register */
-        spin_wait_while_eq( next, (void*)NULL );
+        spin_wait_while_eq( my_next, (void*)NULL );
     }
-    __TBB_ASSERT( next, "still no successor at this point!" );
-    if( next->state & STATE_COMBINED_WAITINGREADER )
-        __TBB_store_with_release(next->going,1);
-    else if( next->state==STATE_UPGRADE_WAITING )
+    scoped_lock *const n = __TBB_load_relaxed(my_next);
+    __TBB_ASSERT( n, "still no successor at this point!" );
+    if( n->my_state & STATE_COMBINED_WAITINGREADER )
+        __TBB_store_with_release(n->my_going,1);
+    else if( n->my_state==STATE_UPGRADE_WAITING )
         // the next waiting for upgrade means this writer was upgraded before.
-        next->state = STATE_UPGRADE_LOSER;
-    state = STATE_ACTIVEREADER;
-
-downgrade_done:
+        n->my_state = STATE_UPGRADE_LOSER;
+    my_state = STATE_ACTIVEREADER;
     return true;
 }
 
 bool queuing_rw_mutex::scoped_lock::upgrade_to_writer()
 {
-    __TBB_ASSERT( state==STATE_ACTIVEREADER, "only active reader can be upgraded" );
+    __TBB_ASSERT( my_state==STATE_ACTIVEREADER, "only active reader can be upgraded" );
 
     queuing_rw_mutex::scoped_lock * tmp;
     queuing_rw_mutex::scoped_lock * me = this;
 
-    ITT_NOTIFY(sync_releasing, mutex);
-    state = STATE_UPGRADE_REQUESTED;
+    ITT_NOTIFY(sync_releasing, my_mutex);
+    my_state = STATE_UPGRADE_REQUESTED;
 requested:
-    __TBB_ASSERT( !( tricky_pointer(next) & FLAG ), "use of corrupted pointer!" );
+    __TBB_ASSERT( !(uintptr_t(__TBB_load_relaxed(my_next)) & FLAG), "use of corrupted pointer!" );
     acquire_internal_lock();
-    if( this != mutex->q_tail.compare_and_swap<tbb::release>(tricky_pointer(me)|FLAG, this) ) {
-        spin_wait_while_eq( next, (void*)NULL );
+    if( this != my_mutex->q_tail.compare_and_swap<tbb::release>(tricky_pointer(me)|FLAG, this) ) {
+        spin_wait_while_eq( my_next, (void*)NULL );
         queuing_rw_mutex::scoped_lock * n;
-        n = tricky_pointer::fetch_and_add<tbb::acquire>(&(this->next), FLAG);
-        unsigned short n_state = n->state;
+        n = tricky_pointer::fetch_and_add<tbb::acquire>(&my_next, FLAG);
+        unsigned short n_state = n->my_state;
         /* the next reader can be blocked by our state. the best thing to do is to unblock it */
         if( n_state & STATE_COMBINED_WAITINGREADER )
-            __TBB_store_with_release(n->going,1);
-        tmp = tricky_pointer::fetch_and_store<tbb::release>(&(n->prev), this);
+            __TBB_store_with_release(n->my_going,1);
+        tmp = tricky_pointer::fetch_and_store<tbb::release>(&(n->my_prev), this);
         unblock_or_wait_on_internal_lock(get_flag(tmp));
         if( n_state & (STATE_COMBINED_READER | STATE_UPGRADE_REQUESTED) ) {
             // save n|FLAG for simplicity of following comparisons
             tmp = tricky_pointer(n)|FLAG;
             atomic_backoff backoff;
-            while(next==tmp) {
-                if( state & STATE_COMBINED_UPGRADING ) {
-                    if( __TBB_load_with_acquire(next)==tmp )
-                        next = n;
+            while(__TBB_load_relaxed(my_next)==tmp) {
+                if( my_state & STATE_COMBINED_UPGRADING ) {
+                    if( __TBB_load_with_acquire(my_next)==tmp )
+                        __TBB_store_relaxed(my_next, n);
                     goto waiting;
                 }
                 backoff.pause();
             }
-            __TBB_ASSERT(next!=(tricky_pointer(n)|FLAG), NULL);
+            __TBB_ASSERT(__TBB_load_relaxed(my_next) != (tricky_pointer(n)|FLAG), NULL);
             goto requested;
         } else {
             __TBB_ASSERT( n_state & (STATE_WRITER | STATE_UPGRADE_WAITING), "unexpected state");
-            __TBB_ASSERT( (tricky_pointer(n)|FLAG)==next, NULL);
-            next = n;
+            __TBB_ASSERT( (tricky_pointer(n)|FLAG) == __TBB_load_relaxed(my_next), NULL);
+            __TBB_store_relaxed(my_next, n);
         }
     } else {
         /* We are in the tail; whoever comes next is blocked by q_tail&FLAG */
         release_internal_lock();
-    } // if( this != mutex->q_tail... )
-    state.compare_and_swap<tbb::acquire>(STATE_UPGRADE_WAITING, STATE_UPGRADE_REQUESTED);
+    } // if( this != my_mutex->q_tail... )
+    my_state.compare_and_swap<tbb::acquire>(STATE_UPGRADE_WAITING, STATE_UPGRADE_REQUESTED);
 
 waiting:
-    __TBB_ASSERT( !( tricky_pointer(next) & FLAG ), "use of corrupted pointer!" );
-    __TBB_ASSERT( state & STATE_COMBINED_UPGRADING, "wrong state at upgrade waiting_retry" );
+    __TBB_ASSERT( !( intptr_t(__TBB_load_relaxed(my_next)) & FLAG ), "use of corrupted pointer!" );
+    __TBB_ASSERT( my_state & STATE_COMBINED_UPGRADING, "wrong state at upgrade waiting_retry" );
     __TBB_ASSERT( me==this, NULL );
-    ITT_NOTIFY(sync_prepare, mutex);
-    /* if noone was blocked by the "corrupted" q_tail, turn it back */
-    mutex->q_tail.compare_and_swap<tbb::release>( this, tricky_pointer(me)|FLAG );
+    ITT_NOTIFY(sync_prepare, my_mutex);
+    /* if no one was blocked by the "corrupted" q_tail, turn it back */
+    my_mutex->q_tail.compare_and_swap<tbb::release>( this, tricky_pointer(me)|FLAG );
     queuing_rw_mutex::scoped_lock * pred;
-    pred = tricky_pointer::fetch_and_add<tbb::acquire>(&(this->prev), FLAG);
+    pred = tricky_pointer::fetch_and_add<tbb::acquire>(&my_prev, FLAG);
     if( pred ) {
         bool success = pred->try_acquire_internal_lock();
-        pred->state.compare_and_swap<tbb::release>(STATE_UPGRADE_WAITING, STATE_UPGRADE_REQUESTED);
+        pred->my_state.compare_and_swap<tbb::release>(STATE_UPGRADE_WAITING, STATE_UPGRADE_REQUESTED);
         if( !success ) {
-            tmp = tricky_pointer::compare_and_swap<tbb::release>(&(this->prev), pred, tricky_pointer(pred)|FLAG );
-            if( tricky_pointer(tmp)&FLAG ) {
-                spin_wait_while_eq(this->prev, pred);
-                pred = this->prev;
+            tmp = tricky_pointer::compare_and_swap<tbb::release>(&my_prev, pred, tricky_pointer(pred)|FLAG );
+            if( uintptr_t(tmp) & FLAG ) {
+                spin_wait_while_eq(my_prev, pred);
+                pred = __TBB_load_relaxed(my_prev);
             } else {
-                spin_wait_while_eq( this->prev, tricky_pointer(pred)|FLAG );
+                spin_wait_while_eq( my_prev, tricky_pointer(pred)|FLAG );
                 pred->release_internal_lock();
             }
         } else {
-            this->prev = pred;
+            __TBB_store_relaxed(my_prev, pred);
             pred->release_internal_lock();
-            spin_wait_while_eq(this->prev, pred);
-            pred = this->prev;
+            spin_wait_while_eq(my_prev, pred);
+            pred = __TBB_load_relaxed(my_prev);
         }
         if( pred )
             goto waiting;
     } else {
-        // restore the corrupted prev field for possible further use (e.g. if downgrade back to reader)
-        this->prev = pred;
+        // restore the corrupted my_prev field for possible further use (e.g. if downgrade back to reader)
+        __TBB_store_relaxed(my_prev, pred);
     }
-    __TBB_ASSERT( !pred && !this->prev, NULL );
+    __TBB_ASSERT( !pred && !__TBB_load_relaxed(my_prev), NULL );
 
     // additional lifetime issue prevention checks
     // wait for the successor to finish working with my fields
     wait_for_release_of_internal_lock();
     // now wait for the predecessor to finish working with my fields
-    spin_wait_while_eq( going, 2 );
-    // there is an acquire semantics statement in the end of spin_wait_while_eq.
+    spin_wait_while_eq( my_going, 2 );
 
-    bool result = ( state != STATE_UPGRADE_LOSER );
-    state = STATE_WRITER;
-    going = 1;
+    // Acquire critical section indirectly from previous owner or directly from predecessor (TODO: not clear).
+    __TBB_control_consistency_helper(); // on either "my_mutex->q_tail" or "my_going" (TODO: not clear)
 
-    ITT_NOTIFY(sync_acquired, mutex);
+    bool result = ( my_state != STATE_UPGRADE_LOSER );
+    my_state = STATE_WRITER;
+    __TBB_store_relaxed(my_going, 1);
+
+    ITT_NOTIFY(sync_acquired, my_mutex);
     return result;
 }
 
