@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2010 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -37,6 +37,10 @@
 #include <cstdio>
 #include <cstdlib>
 
+#if _WIN32||_WIN64
+#include "tbb/machine/windows_api.h"
+#endif
+
 #if !TBB_USE_EXCEPTIONS && _MSC_VER
     // Suppress "C++ exception handler used, but unwind semantics are not enabled" warning in STL headers
     #pragma warning (push)
@@ -166,7 +170,7 @@ void PrintVersion() {
 
 void PrintExtraVersionInfo( const char* category, const char* description ) {
     if( PrintVersionFlag ) 
-        fprintf(stderr, "%s: %s\t%s\n", "TBB", category, description );
+        fprintf(stderr, "TBB: %s\t%s\n", category, description );
 }
 
 void PrintRMLVersionInfo( void* arg, const char* server_info ) {
@@ -218,6 +222,9 @@ extern "C" void __TBB_machine_store8_slow( volatile void *ptr, int64_t value ) {
 #endif /* !__TBB_RML_STATIC */
 
 #if __TBB_ipf
+/* It was found that on IPF inlining of __TBB_machine_lockbyte leads
+   to serious performance regression with ICC 10.0. So keep it out-of-line.
+ */
 extern "C" intptr_t __TBB_machine_lockbyte( volatile unsigned char& flag ) {
     if ( !__TBB_TryLockByte(flag) ) {
         tbb::internal::atomic_backoff b;
