@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2010 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -27,20 +27,11 @@
 */
 
 #include <typeinfo>
+#include "tbb/tbb_exception.h"
 #include "tbb/atomic.h"
 #include "harness.h"
 #include "harness_concurrency_tracker.h"
 
-namespace Harness {
-#if _WIN32 || _WIN64
-    typedef DWORD tid_t;
-    tid_t CurrentTid () { return GetCurrentThreadId(); }
-#else /* !WIN */
-    typedef pthread_t tid_t;
-    tid_t CurrentTid () { return pthread_self(); }
-#endif /* !WIN */
-} // namespace Harness
-
 int g_NumThreads = 0;
 Harness::tid_t  g_Master = 0;
 
