@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2010 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -42,10 +42,10 @@
 
 static bool bad_hashing = false;
 
-namespace tbb { 
+namespace tbb {
     namespace internal {
         static void hooked_warning( const char* /*format*/, ... ) {
-            ASSERT(bad_hashing, "unexpected bad hashing");
+            ASSERT(bad_hashing, "unexpected runtime_warning: bad hashing");
         }
     } // namespace internal
 } // namespace tbb
@@ -75,7 +75,14 @@ namespace tbb {
         size_t hash( UserDefinedKeyType ) const {return 0;}
         bool equal( UserDefinedKeyType /*x*/, UserDefinedKeyType /*y*/ ) {return true;}
     };
-};
+}
+
+#if HARNESS_USE_PROXY
+    #define TBB_PREVIEW_RUNTIME_LOADER 1
+    #include "tbb/runtime_loader.h"
+    static char const * test_path[] = { ".", NULL };
+    static tbb::runtime_loader test_runtime_loader( test_path );
+#endif // HARNESS_USE_PROXY
 
 tbb::concurrent_hash_map<UserDefinedKeyType,int> TestInstantiationWithUserDefinedKeyType;
 
@@ -145,7 +152,7 @@ public:
         --MyDataCount;
         my_state = DEAD;
     }
-    static MyData make( int i ) {   
+    static MyData make( int i ) {
         MyData result;
         result.data = i;
         return result;
@@ -197,7 +204,7 @@ public:
     }
     unsigned long hash( const MyKey& k ) const {
         return k.key;
-    }   
+    }
 };
 
 class YourHashCompare {
@@ -207,7 +214,7 @@ public:
     }
     unsigned long hash( const MyKey& ) const {
         return 1;
-    }   
+    }
 };
 
 typedef local_counting_allocator<std::allocator<MyData> > MyAllocator;
@@ -275,7 +282,7 @@ struct Find {
             else
                 (*ca).second.set_value( ~ca->second.value_of() );
         } else {
-            if( UseKey(i) ) 
+            if( UseKey(i) )
                 REPORT("Line %d: key %d missing\n",__LINE__,i);
         }
     }
@@ -324,7 +331,7 @@ struct InsertErase  {
     static void apply( YourTable& table, int i ) {
         if ( i%3 ) {
             int key = i%IE_SIZE;
-            if ( table.insert( std::make_pair(MyKey::make(key), MyData2()) ) ) 
+            if ( table.insert( std::make_pair(MyKey::make(key), MyData2()) ) )
                 ++InsertEraseCount[key];
         } else {
             int key = i%IE_SIZE;
@@ -354,12 +361,38 @@ struct InnerInsert {
     }
 };
 
+#include "harness_barrier.h"
+// Test for the misuse of constness
+struct FakeExclusive : NoAssign {
+    Harness::SpinBarrier& barrier;
+    YourTable& table;
+    FakeExclusive(Harness::SpinBarrier& b, YourTable&t) : barrier(b), table(t) {}
+    void operator()( int i ) const {
+        if(i) {
+            YourTable::const_accessor real_ca;
+            // const accessor on non-const table aquired as reader (shared)
+            ASSERT( table.find(real_ca,MyKey::make(1)), NULL );
+            barrier.wait(); // item can be erased
+            Harness::Sleep(10); // let it enter the erase
+            real_ca->second.value_of(); // check the state while holding accessor
+        } else {
+            YourTable::accessor fake_ca;
+            const YourTable &const_table = table;
+            // non-const accessor on const table aquired as reader (shared)
+            ASSERT( const_table.find(fake_ca,MyKey::make(1)), NULL );
+            barrier.wait(); // readers aquired
+            // can mistakenly remove the item while other readers still refers to it
+            table.erase( fake_ca );
+        }
+    }
+};
+
 template<typename Op, typename MyTable>
 class TableOperation: NoAssign {
     MyTable& my_table;
 public:
     void operator()( const tbb::blocked_range<int>& range ) const {
-        for( int i=range.begin(); i!=range.end(); ++i ) 
+        for( int i=range.begin(); i!=range.end(); ++i )
             Op::apply(my_table,i);
     }
     TableOperation( MyTable& table ) : my_table(table) {}
@@ -419,14 +452,14 @@ template<typename RangeType>
 struct ParallelTraverseBody: NoAssign {
     const size_t n;
     AtomicByte* const array;
-    ParallelTraverseBody( AtomicByte array_[], size_t n_ ) : 
-        n(n_), 
+    ParallelTraverseBody( AtomicByte array_[], size_t n_ ) :
+        n(n_),
         array(array_)
     {}
     void operator()( const RangeType& range ) const {
         for( typename RangeType::iterator i = range.begin(); i!=range.end(); ++i ) {
             int k = i->first.value_of();
-            ASSERT( 0<=k && size_t(k)<n, NULL ); 
+            ASSERT( 0<=k && size_t(k)<n, NULL );
             ++array[k];
         }
     }
@@ -464,13 +497,13 @@ void ParallelTraverseTable( MyTable& table, size_t n, size_t expected_size ) {
 }
 
 void TestInsertFindErase( int nthread ) {
-    int n=250000; 
+    int n=250000;
 
     // compute m = number of unique keys
-    int m = 0;       
+    int m = 0;
     for( int i=0; i<n; ++i )
         m += UseKey(i);
- 
+
     MyAllocator a; a.items_freed = a.frees = 100;
     ASSERT( MyDataCount==0, NULL );
     MyTable table(a);
@@ -506,12 +539,15 @@ void TestInsertFindErase( int nthread ) {
     if(nthread > 1) {
         YourTable ie_table;
         for( int i=0; i<IE_SIZE; ++i )
-            InsertEraseCount[i] = 0;        
+            InsertEraseCount[i] = 0;
         DoConcurrentOperations<InsertErase,YourTable>(ie_table,n/2,"insert_erase",nthread);
         for( int i=0; i<IE_SIZE; ++i )
             ASSERT( InsertEraseCount[i]==ie_table.count(MyKey::make(i)), NULL );
 
         DoConcurrentOperations<InnerInsert,YourTable>(ie_table,2000,"inner insert",nthread);
+        Harness::SpinBarrier barrier(nthread);
+        REMARK("testing erase on fake exclusive accessor\n");
+        NativeParallelFor( nthread, FakeExclusive(barrier, ie_table));
     }
 }
 
@@ -530,7 +566,7 @@ public:
             while( Counter<i ) {
                 if( ++j==1000000 ) {
                     // If Counter<i after a million iterations, then we almost surely have
-                    // more logical threads than physical threads, and should yield in 
+                    // more logical threads than physical threads, and should yield in
                     // order to let suspended logical threads make progress.
                     j = 0;
                     __TBB_Yield();
@@ -541,13 +577,13 @@ public:
             {
                 MyTable::accessor a;
                 MyKey key = MyKey::make(i);
-                if( my_table.insert( a, key ) ) 
+                if( my_table.insert( a, key ) )
                     a->second.set_value( 1 );
-                else 
-                    a->second.set_value( a->second.value_of()+1 );      
+                else
+                    a->second.set_value( a->second.value_of()+1 );
                 k = a->second.value_of();
             }
-            if( k==my_nthread ) 
+            if( k==my_nthread )
                 Counter=i+1;
         }
     }
@@ -647,7 +683,7 @@ static void FillTable( MyTable& x, int n ) {
     for( int i=1; i<=n; ++i ) {
         MyKey key( MyKey::make(-i) ); // hash values must not be specified in direct order
         typename MyTable::accessor a;
-        bool b = x.insert(a,key); 
+        bool b = x.insert(a,key);
         ASSERT(b, NULL);
         a->second.set_value( i*i );
     }
@@ -661,8 +697,8 @@ static void CheckTable( const MyTable& x, int n ) {
     for( int i=1; i<=n; ++i ) {
         MyKey key( MyKey::make(-i) );
         typename MyTable::const_accessor a;
-        bool b = x.find(a,key); 
-        ASSERT( b, NULL ); 
+        bool b = x.find(a,key);
+        ASSERT( b, NULL );
         ASSERT( a->second.value_of()==i*i, NULL );
     }
     int count = 0;
@@ -717,7 +753,7 @@ void TestAssignment() {
             ASSERT( (t1 == t2) == (i == j), NULL );
             CheckTable(t2,j);
 
-            MyTable& tref = t2=t1; 
+            MyTable& tref = t2=t1;
             ASSERT( &tref==&t2, NULL );
             ASSERT( t1 == t2, NULL );
             CheckTable(t1,i);
@@ -749,7 +785,7 @@ void TestIteratorsAndRanges() {
     TestIteratorAssignment<MyTable::iterator>( v.begin() );
     // doesn't compile as expected: TestIteratorAssignment<typename V::iterator>( u.begin() );
 
-    // check for non-existing 
+    // check for non-existing
     ASSERT(v.equal_range(MyKey::make(-1)) == std::make_pair(v.end(), v.end()), NULL);
     ASSERT(u.equal_range(MyKey::make(-1)) == std::make_pair(u.end(), u.end()), NULL);
 
@@ -892,6 +928,7 @@ int TestMain () {
         REPORT("ERROR: must use at least one thread\n");
         exit(1);
     }
+    if( MaxThread<2 ) MaxThread=2;
 
     // Do serial tests
     TestTypes();
