@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2010 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -26,6 +26,10 @@
     the GNU General Public License.
 */
 
+#include "harness.h"
+
+#if __TBB_TASK_GROUP_CONTEXT
+
 #include <limits.h> // for INT_MAX
 #include "tbb/task_scheduler_init.h"
 #include "tbb/tbb_exception.h"
@@ -39,8 +43,6 @@
 #include "tbb/blocked_range.h"
 #include "harness_assert.h"
 
-#if __TBB_TASK_GROUP_CONTEXT
-
 #define FLAT_RANGE  100000
 #define FLAT_GRAIN  100
 #define NESTING_RANGE  100
@@ -87,10 +89,10 @@ intptr_t TestNumSubrangesCalculation ( intptr_t length, intptr_t grain, intptr_t
 class NoThrowParForBody {
 public:
     void operator()( const range_type& r ) const {
-        volatile long x;
+        volatile count_type x = 0;
         count_type end = r.end();
         for( count_type i=r.begin(); i<end; ++i )
-            x = 0;
+            x += i;
     }
 };
 
@@ -1195,7 +1197,6 @@ void RunPipelineTests() {
     TestCancelation2_pipeline();
 }
 
-#endif /* __TBB_TASK_GROUP_CONTEXT */
 
 #if TBB_USE_EXCEPTIONS
 
@@ -1255,7 +1256,6 @@ int TestMain () {
     MinThread = min(tbb::task_scheduler_init::default_num_threads(), max(2, MinThread));
     MaxThread = max(MinThread, min(tbb::task_scheduler_init::default_num_threads(), MaxThread));
     ASSERT (FLAT_RANGE >= FLAT_GRAIN * MaxThread, "Fix defines");
-#if __TBB_TASK_GROUP_CONTEXT
     int step = max((MaxThread - MinThread + 1)/2, 1);
     for ( g_NumThreads = MinThread; g_NumThreads <= MaxThread; g_NumThreads += step ) {
         REMARK ("Number of threads %d\n", g_NumThreads);
@@ -1275,7 +1275,12 @@ int TestMain () {
     REPORT("Known issue: exception handling tests are skipped.\n");
 #endif
     return Harness::Done;
-#else  /* !__TBB_TASK_GROUP_CONTEXT */
-    return Harness::Skipped;
-#endif /* !__TBB_TASK_GROUP_CONTEXT */
 }
+
+#else /* !__TBB_TASK_GROUP_CONTEXT */
+
+int TestMain () {
+    return Harness::Skipped;
+}
+
+#endif /* !__TBB_TASK_GROUP_CONTEXT */
