@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2010 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -26,13 +26,16 @@
     the GNU General Public License.
 */
 
-#define  __TBB_COUNT_TASK_NODES 1
-#include "harness_inject_scheduler.h"
+#include "harness.h"
 
 #if __TBB_TASK_GROUP_CONTEXT
 
 #define __TBB_ATOMICS_CODEGEN_BROKEN __SUNPRO_CC
 
+#define private public
+#include "tbb/task.h"
+#undef private
+
 #include "tbb/task_scheduler_init.h"
 #include "tbb/spin_mutex.h"
 #include "tbb/tick_count.h"
@@ -241,7 +244,8 @@ class RootLauncherTask : public TaskBase {
  
     tbb::task* do_execute () {
         tbb::task_group_context  ctx(m_CtxKind);
-        SimpleRootTask &r = *new( allocate_root(ctx) ) SimpleRootTask;
+        SimpleRootTask &r = *new( allocate_root() ) SimpleRootTask;
+        r.change_group(ctx);
         TRY();
             spawn_root_and_wait(r);
             // Give a child of our siblings a chance to throw the test exception
@@ -273,6 +277,24 @@ void Test4 () {
     ASSERT_TEST_POSTCOND();
 } // void Test4 ()
 
+/** The same as Test4, except the contexts are bound. **/
+void Test4_1 () {
+    ResetGlobals();
+    tbb::task_list  tl;
+    for ( size_t i = 0; i < NUM_ROOT_TASKS; ++i )
+        tl.push_back( *new( tbb::task::allocate_root() ) RootLauncherTask(tbb::task_group_context::bound) );
+    TRY();
+        tbb::task::spawn_root_and_wait(tl);
+    CATCH_AND_ASSERT();
+    ASSERT (!exceptionCaught, "exception in this scope is unexpected");
+    intptr_t  num_tasks_expected = NUM_ROOT_TASKS * (NUM_CHILD_TASKS + 2);
+    ASSERT (g_CurStat.Existed() == num_tasks_expected, "Wrong total number of tasks");
+    if ( g_SolitaryException )
+        ASSERT (g_CurStat.Executed() >= num_tasks_expected - NUM_CHILD_TASKS, "Unexpected number of executed tasks");
+    ASSERT_TEST_POSTCOND();
+} // void Test4_1 ()
+
+
 class RootsGroupLauncherTask : public TaskBase {
     tbb::task* do_execute () {
         tbb::task_group_context  ctx (tbb::task_group_context::isolated);
@@ -532,7 +554,9 @@ void CheckException () {
     a movable exception in a worker or master thread (depending on the global settings).
     The test also checks the correctness of multiple rethrowing of the pending exception. **/
 void TestMovableException () {
+    REMARK( "TestMovableException\n" );
     ResetGlobals();
+    bool bUnsupported = false;
     tbb::task_group_context ctx;
     tbb::empty_task *r = new( tbb::task::allocate_root() ) tbb::empty_task;
     ASSERT (!g_CurStat.Existing() && !g_CurStat.Existed() && !g_CurStat.Executed(), 
@@ -555,8 +579,13 @@ void TestMovableException () {
             g_ExceptionCaught = true;
             g_UnknownException = unknownException = true;
         }
-        ctx.register_pending_exception();
-        ASSERT (ctx.is_group_execution_cancelled(), "After exception registration the context must be in the cancelled state");
+        try {
+            ctx.register_pending_exception();
+        } catch ( ... ) {
+            bUnsupported = true;
+            REPORT( "Warning: register_pending_exception() failed. This is expected in case of linking with static msvcrt\n" );
+        }
+        ASSERT (ctx.is_group_execution_cancelled() || bUnsupported, "After exception registration the context must be in the cancelled state");
     }
     r->destroy(*r);
     ASSERT_EXCEPTION();
@@ -575,8 +604,8 @@ void TestMovableException () {
         g_ExceptionCaught = true;
         g_UnknownException = true;
     }
-    ASSERT (g_ExceptionCaught, "no exception occurred");
-    ASSERT (__TBB_EXCEPTION_TYPE_INFO_BROKEN || !g_UnknownException, "unknown exception was caught");
+    ASSERT (g_ExceptionCaught || bUnsupported, "no exception occurred");
+    ASSERT (__TBB_EXCEPTION_TYPE_INFO_BROKEN || !g_UnknownException  || bUnsupported, "unknown exception was caught");
     r->destroy(*r);
 } // void Test10 ()
 
@@ -587,7 +616,7 @@ class CtxLauncherTask : public tbb::task {
     tbb::task_group_context &m_Ctx;
 
     tbb::task* execute () {
-        tbb::task::spawn_root_and_wait( *new( tbb::task::allocate_root(m_Ctx) ) T );
+        spawn_root_and_wait( *new( allocate_root(m_Ctx) ) T );
         return NULL;
     }
 public:
@@ -615,7 +644,7 @@ class CtxDestroyerTask : public tbb::task {
     tbb::task* execute () {
         ASSERT ( m_nestingLevel >= 0 && m_nestingLevel < MaxNestingDepth, "Wrong nesting level. The test is broken" );
         tbb::task_group_context  ctx;
-        tbb::task *t = new( tbb::task::allocate_root(ctx) ) tbb::empty_task;
+        tbb::task *t = new( allocate_root(ctx) ) tbb::empty_task;
         int level = ++m_nestingLevel;
         if ( level < MaxNestingDepth ) {
             execute();
@@ -642,6 +671,7 @@ int CtxDestroyerTask::s_numCancelled = 0;
 //! Test for data race between cancellation propagation and context destruction.
 /** If the data race ever occurs, an assertion inside TBB will be triggered. **/
 void TestCtxDestruction () {
+    REMARK( "TestCtxDestruction\n" );
     for ( size_t i = 0; i < 10; ++i ) {
         tbb::task_group_context  ctx;
         tbb::task_list  tl;
@@ -729,6 +759,7 @@ void RunTests () {
     Test2();
     Test3();
     Test4();
+    Test4_1();
     Test5();
     Test6();
     Test7();
@@ -741,14 +772,12 @@ void RunTests () {
     TestConcurrentCtxDestruction();
 #endif
 }
-#endif /* __TBB_TASK_GROUP_CONTEXT */
 
 int TestMain () {
-    REMARK ("Using %s", TBB_USE_CAPTURED_EXCEPTION ? "tbb:captured_exception" : "exact exception propagation");
+    REMARK ("Using %s\n", TBB_USE_CAPTURED_EXCEPTION ? "tbb:captured_exception" : "exact exception propagation");
     MinThread = min(NUM_ROOTS_IN_GROUP, min(tbb::task_scheduler_init::default_num_threads(), max(2, MinThread)));
     MaxThread = min(NUM_ROOTS_IN_GROUP, max(MinThread, min(tbb::task_scheduler_init::default_num_threads(), MaxThread)));
     ASSERT (NUM_ROOTS_IN_GROUP < NUM_ROOT_TASKS, "Fix defines");
-#if __TBB_TASK_GROUP_CONTEXT
 #if TBB_USE_EXCEPTIONS
     // Test0 always runs on one thread
     Test0();
@@ -757,7 +786,12 @@ int TestMain () {
     for ( g_NumThreads = MinThread; g_NumThreads <= MaxThread; ++g_NumThreads )
         RunTests();
     return Harness::Done;
-#else
-    return Harness::Skipped;
-#endif /* __TBB_TASK_GROUP_CONTEXT */
 }
+
+#else /* !__TBB_TASK_GROUP_CONTEXT */
+
+int TestMain () {
+    return Harness::Skipped;
+}
+
+#endif /* !__TBB_TASK_GROUP_CONTEXT */
