@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2010 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -56,7 +56,7 @@
 #include "harness_assert.h"
 #include "harness.h"
 
-#if __TBB_GCC_WARNING_SUPPRESSION_ENABLED
+#if __TBB_GCC_WARNING_SUPPRESSION_PRESENT
 #pragma GCC diagnostic ignored "-Wuninitialized"
 #endif
 
@@ -136,8 +136,7 @@ struct FunctorAddCombineRef<minimal<N> > {
 static tbb::atomic<int> FinitCounter;
 
 template <typename T, int Value>
-class FunctorFinit {
-public:
+struct FunctorFinit {
     FunctorFinit( const FunctorFinit& ) {++FinitCounter;}
     FunctorFinit( SecretTagType ) {++FinitCounter;}
     ~FunctorFinit() {--FinitCounter;}
@@ -889,6 +888,7 @@ flog_segmented_iterator_map() {
            REMARK( "ii=%d, (*my_csi).first=%d, second=%d\n",ii, int((*my_csi).first), int((*my_csi).second));
        }
    }
+   if(found_error) REPORT("segmented_iterator_map failed\n");
 }
 
 void
@@ -1020,4 +1020,3 @@ int TestMain () {
 
     return Harness::Done;
 }
-
