@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2010 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -26,20 +26,12 @@
     the GNU General Public License.
 */
 
-#include "tbb/task.h"
+#include "harness_task.h"
 #include "tbb/atomic.h"
 #include "tbb/tbb_thread.h"
-#include "harness_assert.h"
+#include "tbb/task_scheduler_init.h"
 #include <cstdlib>
 
-//------------------------------------------------------------------------
-// Helper for verifying that old use cases of spawn syntax still work.
-//------------------------------------------------------------------------
-tbb::task* GetTaskPtr( int& counter ) {
-    ++counter;
-    return NULL;
-}
- 
 //------------------------------------------------------------------------
 // Test for task::spawn_children and task_list
 //------------------------------------------------------------------------
@@ -71,7 +63,7 @@ public:
                 // We are stolen, let our parent to start waiting for us
                 m_Parent->m_GoAhead = false;
             }
-            tbb::task &t = *new( tbb::task::allocate_child() ) this_type(this, m_Depth - 1);
+            tbb::task &t = *new( allocate_child() ) this_type(this, m_Depth - 1);
             set_ref_count( 2 );
             spawn( t );
             // Give a willing thief a chance to steal
@@ -114,14 +106,14 @@ public:
             tbb::task_list list;
             ASSERT( list.empty(), NULL );
             for( int k=0; k<m_ChildCount; ++k ) {
-                list.push_back( *new( tbb::task::allocate_child() ) RecursiveTask(m_ChildCount/2,m_Depth-1 ) );
+                list.push_back( *new( allocate_child() ) RecursiveTask(m_ChildCount/2,m_Depth-1 ) );
                 ASSERT( !list.empty(), NULL );
             }
             set_ref_count( m_ChildCount+1 );
             SpawnList( list, 1 );
             // Now try reusing this as the parent.
             set_ref_count(2);
-            list.push_back( *new (tbb::task::allocate_child() ) tbb::empty_task() );
+            list.push_back( *new ( allocate_child() ) tbb::empty_task() );
             SpawnList( list, 2 );
         }
         return NULL;
@@ -133,9 +125,6 @@ static int Expected( int child_count, int depth ) {
     return depth<=0 ? 1 : 1+child_count*Expected(child_count/2,depth-1);
 }
 
-#include "tbb/task_scheduler_init.h"
-#include "harness.h"
-
 void TestStealLimit( int nthread ) {
     REMARK( "testing steal limiting heuristics for %d threads\n", nthread );
     tbb::task_scheduler_init init(nthread);
@@ -176,33 +165,6 @@ void TestSpawnRootList( int nthread ) {
 // Test for task::recycle_as_safe_continuation
 //------------------------------------------------------------------------
 
-class TaskGenerator: public tbb::task {
-    int m_ChildCount;
-    int m_Depth;
-    
-public:
-    TaskGenerator( int child_count, int _depth ) : m_ChildCount(child_count), m_Depth(_depth) {}
-    ~TaskGenerator( ) { m_ChildCount = m_Depth = -125; }
-
-    /*override*/ tbb::task* execute() {
-        ASSERT( m_ChildCount>=0 && m_Depth>=0, NULL );
-        if( m_Depth>0 ) {
-            recycle_as_safe_continuation();
-            set_ref_count( m_ChildCount+1 );
-            int k=0; 
-            for( int j=0; j<m_ChildCount; ++j ) {
-                tbb::task& t = *new( allocate_child() ) TaskGenerator(m_ChildCount/2,m_Depth-1);
-                GetTaskPtr(k)->spawn(t);
-            }
-            ASSERT(k==m_ChildCount,NULL);
-            --m_Depth;
-            __TBB_Yield();
-            ASSERT( state()==recycle && ref_count()>0, NULL);
-        }
-        return NULL;
-    }
-};
-
 void TestSafeContinuation( int nthread ) {
     REMARK("testing task::recycle_as_safe_continuation for %d threads\n",nthread);
     tbb::task_scheduler_init init(nthread);
@@ -218,12 +180,12 @@ void TestSafeContinuation( int nthread ) {
 tbb::atomic<int> TotalCount;
 
 struct AffinityTask: public tbb::task {
-    const tbb::task::affinity_id expected_affinity_id; 
+    const affinity_id expected_affinity_id; 
     bool noted;
     /** Computing affinities is NOT supported by TBB, and may disappear in the future.
         It is done here for sake of unit testing. */
     AffinityTask( int expected_affinity_id_ ) : 
-        expected_affinity_id(tbb::task::affinity_id(expected_affinity_id_)), 
+        expected_affinity_id(affinity_id(expected_affinity_id_)), 
         noted(false) 
     {
         set_affinity(expected_affinity_id);
@@ -241,7 +203,7 @@ struct AffinityTask: public tbb::task {
         // We exploit those conditions for sake of unit testing.
         ASSERT( id!=expected_affinity_id, NULL );
         ASSERT( !noted, "note_affinity_id called twice!" );
-        ASSERT ( &tbb::task::self() == (tbb::task*)this, "Wrong innermost running task" );
+        ASSERT ( &self() == (tbb::task*)this, "Wrong innermost running task" );
         noted = true;
     }
 };
@@ -278,7 +240,7 @@ struct NoteAffinityTask: public tbb::task {
     bool noted;
     NoteAffinityTask( int id ) : noted(false)
     {
-        set_affinity(tbb::task::affinity_id(id));
+        set_affinity(affinity_id(id));
     }
     ~NoteAffinityTask () {
         ASSERT (noted, "note_affinity has not been called");
@@ -288,7 +250,7 @@ struct NoteAffinityTask: public tbb::task {
     }
     /*override*/ void note_affinity( affinity_id /*id*/ ) {
         noted = true;
-        ASSERT ( &tbb::task::self() == (tbb::task*)this, "Wrong innermost running task" );
+        ASSERT ( &self() == (tbb::task*)this, "Wrong innermost running task" );
     }
 };
 
@@ -394,7 +356,7 @@ class TaskWithMember: public tbb::task {
         x = y;
         if( count>0 ) { 
             set_ref_count(2);
-            tbb::task* t = new( tbb::task::allocate_child() ) TaskWithMember<T>(count-1);
+            tbb::task* t = new( allocate_child() ) TaskWithMember<T>(count-1);
             spawn_and_wait_for_all(*t);
         }
         return NULL;
@@ -429,7 +391,10 @@ void TestAlignment() {
     TestAlignmentOfOneClass<double>();
 #if HAVE_m128
     TestAlignmentOfOneClass<__m128>();
-#endif /* HAVE_m128 */
+#endif
+#if HAVE_m256
+    if (have_AVX()) TestAlignmentOfOneClass<__m256>();
+#endif
 }
 
 //------------------------------------------------------------------------
@@ -494,7 +459,7 @@ public:
     DagTask *successor_to_below, *successor_to_right;
     DagTask( int i_, int j_ ) : i(i_), j(j_), sum_from_left(0), sum_from_above(0) {}
     task* execute() {
-        __TBB_ASSERT( ref_count()==0, NULL );
+        ASSERT( ref_count()==0, NULL );
         number_t sum = i==0 && j==0 ? 1 : sum_from_left+sum_from_above;
         check_sum(sum);
         ++execution_count;
@@ -555,7 +520,7 @@ class RelaxedOwnershipTask: public tbb::task {
 
     tbb::task* execute () {
         tbb::task &p = *parent();
-        tbb::task &r = *new( tbb::task::allocate_root() ) tbb::empty_task;
+        tbb::task &r = *new( allocate_root() ) tbb::empty_task;
         r.set_ref_count( 1 );
         m_barrier.wait();
         p.spawn( *new(p.allocate_child()) tbb::empty_task );
@@ -625,7 +590,6 @@ void TestUserThread( int p ) {
     }
 }
 
-
 class TaskWithChildToSteal : public tbb::task {
     const int m_Depth; 
     volatile bool m_GoAhead;
@@ -640,7 +604,7 @@ public:
     tbb::task* execute() {
         m_GoAhead = true;
         if ( m_Depth > 0 ) {
-            TaskWithChildToSteal &t = *new( tbb::task::allocate_child() ) TaskWithChildToSteal(m_Depth - 1);
+            TaskWithChildToSteal &t = *new( allocate_child() ) TaskWithChildToSteal(m_Depth - 1);
             t.SpawnAndWaitOnParent();
         }
         else
@@ -657,6 +621,7 @@ public:
     }
 }; // TaskWithChildToSteal
 
+// Success criterion of this test is not hanging
 void TestDispatchLoopResponsiveness() {
     REMARK("testing that dispatch loops do not go into eternal sleep when all remaining children are stolen\n");
     // Recursion depth values test the following sorts of dispatch loops
@@ -670,7 +635,6 @@ void TestDispatchLoopResponsiveness() {
         t.SpawnAndWaitOnParent();
     }
     r.destroy(r);
-    // The success criteria of this test is not hanging
 }
 
 void TestWaitDiscriminativenessWithoutStealing() {
@@ -763,14 +727,14 @@ struct MasterBody : NoAssign, Harness::NoAfterlife {
             if( !m_Depth )
                 return NULL;
             set_ref_count(3);
-            spawn( *new( tbb::task::allocate_child() ) BinaryRecursiveTask(m_Depth - 1) );
-            spawn( *new( tbb::task::allocate_child() ) BinaryRecursiveTask(m_Depth - 1) );
+            spawn( *new( allocate_child() ) BinaryRecursiveTask(m_Depth - 1) );
+            spawn( *new( allocate_child() ) BinaryRecursiveTask(m_Depth - 1) );
             wait_for_all();
             return NULL;
         }
 
         void note_affinity( affinity_id ) {
-            __TBB_ASSERT( false, "These tasks cannot be stolen" );
+            ASSERT( false, "These tasks cannot be stolen" );
         }
     public:
         BinaryRecursiveTask ( int depth_ ) : m_Depth(depth_) {}
@@ -815,143 +779,6 @@ void TestMastersIsolation ( int p ) {
     }
 }
 
-//------------------------------------------------------------------------
-// Test for tbb::task::enqueue
-//------------------------------------------------------------------------
-
-const int PairsPerTrack = 100;
-
-class EnqueuedTask : public tbb::task {
-    task* my_successor;
-    int my_enqueue_order;
-    int* my_track;
-    tbb::task* execute() {
-        // Capture execution order in the very beginning
-        int execution_order = 2 - my_successor->decrement_ref_count();
-        // Create some local work.
-        TaskGenerator& p = *new( tbb::task::allocate_root() ) TaskGenerator(2,2);
-        tbb::task::spawn_root_and_wait(p);
-        if( execution_order==2 ) { // the "slower" of two peer tasks
-            ++nCompletedPairs;
-            // Of course execution order can differ from dequeue order.
-            // But there is no better approximation at hand; and a single worker
-            // will execute in dequeue order, which is enough for our check.
-            if (my_enqueue_order==execution_order)
-                ++nOrderedPairs;
-            FireTwoTasks(my_track);
-            destroy(*my_successor);
-        }
-        return NULL;
-    }
-public:
-    EnqueuedTask( task* successor, int enq_order, int* track )
-    : my_successor(successor), my_enqueue_order(enq_order), my_track(track) {}
-
-    // Create and enqueue two tasks
-    static void FireTwoTasks( int* track ) {
-        int progress = ++*track;
-        if( progress < PairsPerTrack ) {
-            task* successor = new (tbb::task::allocate_root()) tbb::empty_task;
-            successor->set_ref_count(2);
-            enqueue( *new (tbb::task::allocate_root()) EnqueuedTask(successor, 1, track) );
-            enqueue( *new (tbb::task::allocate_root()) EnqueuedTask(successor, 2, track) );
-        }
-    }
-
-    static tbb::atomic<int> nCompletedPairs;
-    static tbb::atomic<int> nOrderedPairs;
-};
-
-tbb::atomic<int> EnqueuedTask::nCompletedPairs;
-tbb::atomic<int> EnqueuedTask::nOrderedPairs;
-
-const int nTracks = 10;
-static int TaskTracks[nTracks];
-const int stall_threshold = 100000;
-
-void TimedYield( double pause_time );
-
-class ProgressMonitor {
-public:
-    void operator() ( ) {
-        int track_snapshot[nTracks];
-        int stall_count = 0, uneven_progress_count = 0, last_progress_mask = 0;
-        for(int i=0; i<nTracks; ++i)
-            track_snapshot[i]=0;
-        bool completed;
-        do {
-            // Yield repeatedly for at least 1 usec
-            TimedYield( 1E-6 );
-            int overall_progress = 0, progress_mask = 0;
-            const int all_progressed = (1<<nTracks) - 1;
-            completed = true;
-            for(int i=0; i<nTracks; ++i) {
-                int ti = TaskTracks[i];
-                int pi = ti-track_snapshot[i];
-                if( pi ) progress_mask |= 1<<i;
-                overall_progress += pi;
-                completed = completed && ti==PairsPerTrack;
-                track_snapshot[i]=ti;
-            }
-            // The constants in the next asserts are subjective and may need correction.
-            if( overall_progress )
-                stall_count=0;
-            else {
-                ++stall_count;
-                // no progress for at least 0.1 s; consider it dead.
-                ASSERT(stall_count < stall_threshold, "no progress on enqueued tasks; deadlock?");
-            }
-            if( progress_mask==all_progressed || progress_mask^last_progress_mask ) {
-                uneven_progress_count = 0;
-                last_progress_mask = progress_mask;
-            }
-            else if ( overall_progress > 2 ) {
-                ++uneven_progress_count;
-                ASSERT(uneven_progress_count < 5, "some enqueued tasks seem stalling; no simultaneous progress?");
-            }
-        } while( !completed );
-    }
-};
-
-void TestEnqueue( int p ) {
-    REMARK("testing task::enqueue for %d threads\n", p);
-    for(int mode=0;mode<3;++mode) {
-        tbb::task_scheduler_init init(p);
-        EnqueuedTask::nCompletedPairs = EnqueuedTask::nOrderedPairs = 0;
-        for(int i=0; i<nTracks; ++i) {
-            TaskTracks[i] = -1; // to accomodate for the starting call
-            EnqueuedTask::FireTwoTasks(TaskTracks+i);
-        }
-        ProgressMonitor pm;
-        tbb::tbb_thread thr( pm );
-        if(mode==1) {
-            // do some parallel work in the meantime
-            for(int i=0; i<10; i++) {
-                TaskGenerator& g = *new( tbb::task::allocate_root() ) TaskGenerator(2,5);
-                tbb::task::spawn_root_and_wait(g);
-                TimedYield( 1E-6 );
-            }
-        }
-        if( mode==2 ) {
-            // Additionally enqueue a bunch of empty tasks. The goal is to test that tasks
-            // allocated and enqueued by a thread are safe to use after the thread leaves TBB.
-            tbb::task* root = new (tbb::task::allocate_root()) tbb::empty_task;
-            root->set_ref_count(100);
-            for( int i=0; i<100; ++i )
-                tbb::task::enqueue( *new (root->allocate_child()) tbb::empty_task );
-            init.terminate(); // master thread deregistered
-        }
-        thr.join();
-        ASSERT(EnqueuedTask::nCompletedPairs==nTracks*PairsPerTrack, NULL);
-        ASSERT(EnqueuedTask::nOrderedPairs<EnqueuedTask::nCompletedPairs,
-            "all task pairs executed in enqueue order; de facto guarantee is too strong?");
-    }
-}
-
-//------------------------------------------------------------------------
-// Run all tests.
-//------------------------------------------------------------------------
-
 int TestMain () {
 #if TBB_USE_EXCEPTIONS
     TestUnconstructibleTask<1>();
@@ -966,24 +793,13 @@ int TestMain () {
         TestSpawnChildren( p );
         TestSpawnRootList( p );
         TestSafeContinuation( p );
-        TestEnqueue( p );
         TestLeftRecursion( p );
         TestDag( p );
         TestAffinity( p );
         TestUserThread( p );
         TestStealLimit( p );
         TestRelaxedOwnership( p );
-#if __TBB_ARENA_PER_MASTER
         TestMastersIsolation( p );
-#endif /* __TBB_ARENA_PER_MASTER */
     }
     return Harness::Done;
 }
-
-#include "tbb/tick_count.h"
-void TimedYield( double pause_time ) {
-    tbb::tick_count start = tbb::tick_count::now();
-    while( (tbb::tick_count::now()-start).seconds() < pause_time )
-        __TBB_Yield();
-}
-
