@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2010 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -38,6 +38,21 @@
     switching producer thread, and the check is repeated.
 */
 
+#if HARNESS_USE_PROXY
+
+// The test includes injects scheduler directly, so skip it when proxy tested.
+
+#undef HARNESS_USE_PROXY
+#include "harness.h"
+#undef __TBB_DYNAMIC_LOAD_ENABLED
+#include "harness_tbb_independence.h"
+
+int TestMain () {
+    return Harness::Skipped;
+}
+
+#else // HARNESS_USE_PROXY
+
 #define  __TBB_COUNT_TASK_NODES 1
 #include "harness_inject_scheduler.h"
 
@@ -92,7 +107,7 @@ public:
         if( my_depth>0 ) {
             int child_count = my_child_count;
             scheduler* my_sched = internal::governor::local_scheduler();
-            tbb::task& c  = *new( tbb::task::allocate_continuation() ) tbb::empty_task;
+            tbb::task& c  = *new( allocate_continuation() ) tbb::empty_task;
             c.set_ref_count( child_count );
             recycle_as_child_of(c);
             --child_count;
@@ -140,9 +155,9 @@ void RunTaskGenerators( bool switchProducer = false, bool checkProducer = false
     dummy_root->set_ref_count( 2 );
     // If no producer, start elections; some worker will take the role
     if( Producer )
-        dummy_root->spawn( *new( dummy_root->allocate_child() ) tbb::empty_task );
+        tbb::task::spawn( *new( dummy_root->allocate_child() ) tbb::empty_task );
     else
-        dummy_root->spawn( *new( dummy_root->allocate_child() ) ChangeProducer );
+        tbb::task::spawn( *new( dummy_root->allocate_child() ) ChangeProducer );
     if( checkProducer && !Producer )
         REPORT("Warning: producer has not changed after 10 attempts; running on a single core?\n");
     for( int j=0; j<100; ++j ) {
@@ -150,13 +165,46 @@ void RunTaskGenerators( bool switchProducer = false, bool checkProducer = false
             tbb::task& t = *new( tbb::task::allocate_root() ) TaskGenerator(/*child_count=*/4, /*depth=*/6);
             tbb::task::spawn_root_and_wait(t);
         } else {
-            tbb::task& t = *new (dummy_root->allocate_additional_child_of(*dummy_root))
+            tbb::task& t = *new (tbb::task::allocate_additional_child_of(*dummy_root))
                                 TaskGenerator(/*child_count=*/4, /*depth=*/6);
             tbb::task::enqueue(t);
         }
     }
     dummy_root->wait_for_all();
-    dummy_root->destroy( *dummy_root );
+    tbb::task::destroy( *dummy_root );
+}
+
+class TaskList: public tbb::task {
+    const int my_num_childs;
+public:
+    TaskList(const int num_childs) : my_num_childs(num_childs) {}
+    tbb::task* execute() {
+        tbb::task_list list;
+        for (int i=0; i<my_num_childs; ++i)
+        {
+            list.push_back( *new( allocate_child() ) tbb::empty_task );
+        }
+        set_ref_count(my_num_childs+1);
+        spawn(list);
+
+        wait_for_all();
+        return 0;
+    }
+};
+
+void RunTaskListGenerator()
+{
+    const int max_num_childs = 10000;
+    int num_childs=3;
+
+    while ( num_childs < max_num_childs )
+    {
+        tbb::task& root = *new( tbb::task::allocate_root() ) TaskList(num_childs);
+
+        tbb::task::spawn_root_and_wait(root);
+
+        num_childs = 3 * num_childs;
+    }
 }
 
 //! Tests whether task scheduler allows thieves to hoard task objects.
@@ -179,6 +227,7 @@ void TestTaskReclamation() {
     for( int i=0; i<N; ++i ) {
         // First N iterations fill internal buffers and collect initial statistics
         RunTaskGenerators();
+        RunTaskListGenerator();
 
         size_t m = GetMemoryUsage();
         if( m-initial_amount_of_memory > 0)
@@ -203,6 +252,7 @@ void TestTaskReclamation() {
     for( int i=0; i < MaxIterations; ++i ) {
         // These iterations check for excessive memory use and unreasonable task count
         RunTaskGenerators( switchProducer, checkProducer );
+        RunTaskListGenerator();
 
         intptr_t n = internal::governor::local_scheduler()->get_task_node_count( /*count_arena_workers=*/true );
         size_t m = GetMemoryUsage();
@@ -241,3 +291,5 @@ int TestMain () {
     return Harness::Done;
 }
 
+#endif  // HARNESS_USE_PROXY
+
